package com.book.dao.config;


import org.apache.ibatis.session.SqlSessionFactory;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.mybatis.spring.SqlSessionTemplate;
import org.mybatis.spring.annotation.MapperScan;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.jdbc.DataSourceBuilder;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;

import javax.sql.DataSource;

@Configuration
@MapperScan(basePackages = "com.book.dao.cps.mapper", sqlSessionTemplateRef = "cpsSqlSessionTemplate")
public class CpsDataSourceConfig{

    @Bean(name = "cpsDataSource")
    @ConfigurationProperties(prefix = "spring.datasource.cps")
    @Primary
    public DataSource statementDataSource() {
        return DataSourceBuilder.create().build();
    }

    @Bean(name = "cpsSqlSessionFactory")
    @Primary
    public SqlSessionFactory statementSqlSessionFactory(@Qualifier("cpsDataSource") DataSource dataSource) throws Exception {
        SqlSessionFactoryBean bean = new SqlSessionFactoryBean();
        bean.setDataSource(dataSource);
        bean.setMapperLocations(new PathMatchingResourcePatternResolver().getResources("classpath:mapper/cps/*.xml"));
        return bean.getObject();
    }

    @Bean(name = "cpsTransactionManager")
    @Primary
    public DataSourceTransactionManager statementTransactionManager(@Qualifier("cpsDataSource") DataSource dataSource) {
        return new DataSourceTransactionManager(dataSource);
    }

    @Bean(name = "cpsSqlSessionTemplate")
    @Primary
    public SqlSessionTemplate statementSqlSessionTemplate(@Qualifier("cpsSqlSessionFactory") SqlSessionFactory sqlSessionFactory) throws Exception {
        return new SqlSessionTemplate(sqlSessionFactory);
    }
}
