package com.book.dao.config;

import org.apache.ibatis.session.SqlSessionFactory;
import org.mybatis.spring.SqlSessionFactoryBean;
import org.mybatis.spring.SqlSessionTemplate;
import org.mybatis.spring.annotation.MapperScan;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.boot.jdbc.DataSourceBuilder;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.core.io.support.PathMatchingResourcePatternResolver;
import org.springframework.jdbc.datasource.DataSourceTransactionManager;

import javax.sql.DataSource;

@Configuration
@MapperScan(basePackages = "com.book.dao.polardb.mapper", sqlSessionTemplateRef  = "polarSqlSessionTemplate")
public class PolarDataSourceConfig {

    @Bean(name = "polarDataSource")
    @ConfigurationProperties(prefix = "spring.datasource.polar")
    public DataSource polarDataSource() {
        return DataSourceBuilder.create().build();
    }

    @Bean(name = "polarSqlSessionFactory")
    public SqlSessionFactory polarSqlSessionFactory(@Qualifier("polarDataSource") DataSource dataSource) throws Exception {
        SqlSessionFactoryBean bean = new SqlSessionFactoryBean();
        bean.setDataSource(dataSource);
         bean.setMapperLocations(new PathMatchingResourcePatternResolver().getResources("classpath:mapper/polar/*.xml"));
       return bean.getObject();
    }

    @Bean(name = "polarTransactionManager")
    public DataSourceTransactionManager polarTransactionManager(@Qualifier("polarDataSource") DataSource dataSource) {
        return new DataSourceTransactionManager(dataSource);
    }

    @Bean(name = "polarSqlSessionTemplate")
    public SqlSessionTemplate polarSqlSessionTemplate(@Qualifier("polarSqlSessionFactory") SqlSessionFactory sqlSessionFactory) throws Exception {
        return new SqlSessionTemplate(sqlSessionFactory);
    }
}
