package com.book.push.controller;

import com.book.dao.cps.pojo.AdminConfig;
import com.book.dao.cps.pojo.Platform;
import com.book.dao.utils.DateUtils;
import com.book.push.service.dao.AdminConfigService;
import com.book.push.service.dao.PlatformService;
import com.book.push.service.dao.UserService;
import com.book.push.service.wx.WxThirdPartService;
import me.chanjar.weixin.common.error.WxErrorException;
import me.chanjar.weixin.mp.bean.kefu.WxMpKefuMessage;
import me.chanjar.weixin.mp.bean.message.WxMpXmlMessage;
import me.chanjar.weixin.mp.bean.message.WxMpXmlOutMessage;
import me.chanjar.weixin.open.api.WxOpenService;
import me.chanjar.weixin.open.bean.message.WxOpenXmlMessage;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * created in 2021/8/26
 * Project: book-store
 *
 * @author win7
 */
@RestController
@RequestMapping("/api/wechat")
public class WxEventController {

    public static final Logger logger = LoggerFactory.getLogger(WxEventController.class);
    @Autowired
    private WxThirdPartService thirdPartService;
    @Autowired
    private PlatformService platformService;
    @Autowired
    private UserService userService;
    @Autowired
    private AdminConfigService adminConfigService;

    @RequestMapping("/mpapi/appid/{appId}")
    public Object callback(@RequestBody(required = false) String requestBody,
                           @PathVariable("appId") String appId,
                           @RequestParam("signature") String signature,
                           @RequestParam("timestamp") String timestamp,
                           @RequestParam("nonce") String nonce,
                           @RequestParam("openid") String openid,
                           @RequestParam("encrypt_type") String encType,
                           @RequestParam("msg_signature") String msgSignature) {
        logger.info(
                "\n接收微信请求：[appId=[{}], openid=[{}], signature=[{}], encType=[{}], msgSignature=[{}],"
                        + " timestamp=[{}], nonce=[{}], requestBody=[\n{}\n] ",
                appId, openid, signature, encType, msgSignature, timestamp, nonce, requestBody);


//        Platform platform = platformService.selectByAppid(appId);
        AdminConfig adminConfig = adminConfigService.selectByAppid(appId);
        Integer platformId = adminConfig.getPlatformId();

        WxOpenService wxOpenService = thirdPartService.getWxOpenServiceByPlatFormId(platformId);

        if (!StringUtils.equalsIgnoreCase("aes", encType)
                || !wxOpenService.getWxOpenComponentService().checkSignature(timestamp, nonce, signature)) {
            throw new IllegalArgumentException("非法请求，可能属于伪造的请求！");
        }

        String out = "";
        // aes加密的消息
        WxMpXmlMessage inMessage = WxOpenXmlMessage.fromEncryptedMpXml(requestBody,
                wxOpenService.getWxOpenConfigStorage(), timestamp, nonce, msgSignature);
        logger.debug("\n消息解密后内容为：\n{} ", inMessage.toString());
        // 全网发布测试用例
        if (StringUtils.equalsAnyIgnoreCase(appId, "wxd101a85aa106f53e", "wx570bc396a51b8ff8")) {
            try {
                out = AllNetTest(appId, wxOpenService, out, inMessage);
            } catch (WxErrorException e) {
                logger.error("测试报错：", e);
            }
        } else {
            WxMpXmlOutMessage outMessage = thirdPartService.getWxThirdPartMessageRouter().route(inMessage, platformId, appId);
            //todo update user operate time
            userService.updateOperateTimeByOpenid(inMessage.getFromUser(), DateUtils.getNow());
            if (outMessage != null) {
                out = WxOpenXmlMessage.wxMpOutXmlMessageToEncryptedXml(outMessage, wxOpenService.getWxOpenConfigStorage());
            }
        }
        return out;
    }


    /**
     * 全网测试
     *
     * @param appId
     * @param wxOpenService
     * @param out
     * @param inMessage
     * @return
     * @throws WxErrorException
     */
    private String AllNetTest(String appId, WxOpenService wxOpenService, String out, WxMpXmlMessage inMessage) throws WxErrorException {
        if (StringUtils.equals(inMessage.getMsgType(), "text")) {
            if (StringUtils.equals(inMessage.getContent(), "TESTCOMPONENT_MSG_TYPE_TEXT")) {
                out = WxOpenXmlMessage.wxMpOutXmlMessageToEncryptedXml(
                        WxMpXmlOutMessage.TEXT().content("TESTCOMPONENT_MSG_TYPE_TEXT_callback")
                                .fromUser(inMessage.getToUser())
                                .toUser(inMessage.getFromUser())
                                .build(),
                        wxOpenService.getWxOpenConfigStorage()
                );
            } else if (StringUtils.startsWith(inMessage.getContent(), "QUERY_AUTH_CODE:")) {
                String msg = inMessage.getContent().replace("QUERY_AUTH_CODE:", "") + "_from_api";
                WxMpKefuMessage kefuMessage = WxMpKefuMessage.TEXT().content(msg).toUser(inMessage.getFromUser()).build();
                wxOpenService.getWxOpenComponentService().getWxMpServiceByAppid(appId).getKefuService().sendKefuMessage(kefuMessage);
            }
        } else if (StringUtils.equals(inMessage.getMsgType(), "event")) {
            WxMpKefuMessage kefuMessage = WxMpKefuMessage.TEXT().content(inMessage.getEvent() + "from_callback").toUser(inMessage.getFromUser()).build();
            wxOpenService.getWxOpenComponentService().getWxMpServiceByAppid(appId).getKefuService().sendKefuMessage(kefuMessage);
        }
        return out;
    }


}
