package com.book.push.controller;

import com.book.dao.cps.pojo.*;
import com.book.dao.cps.pojo.Ophost;
import com.book.dao.polardb.pojo.UserSilent;
import com.book.push.cons.SchemeCons;
import com.book.push.cons.SiteCons;
import com.book.push.cons.UrlCons;
import com.book.push.service.dao.UserSilentService;
import com.book.push.service.dao.AdminConfigService;
import com.book.push.service.dao.ConfigService;
import com.book.push.service.dao.OpHostService;
import com.book.push.service.dao.UserService;
import com.book.push.service.redis.RedisService;
import com.book.push.service.wx.WxThirdPartService;
import com.book.push.utils.JsonUtils;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.api.WxConsts;
import me.chanjar.weixin.common.bean.WxOAuth2UserInfo;
import me.chanjar.weixin.common.bean.oauth2.WxOAuth2AccessToken;
import me.chanjar.weixin.common.error.WxErrorException;
import me.chanjar.weixin.mp.bean.result.WxMpUser;
import me.chanjar.weixin.open.api.WxOpenMpService;
import me.chanjar.weixin.open.api.WxOpenService;
import org.apache.ibatis.annotations.Param;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

/**
 * 接收微信重定向
 *
 * @author win7
 */
@Controller
@Slf4j
@RequestMapping("/wx/redirect")
public class WxRedirectController {

    @Autowired
    private UserService userService;
    @Autowired
    private AdminConfigService adminConfigService;
    @Autowired
    private WxThirdPartService thirdPartService;
    @Autowired
    private ConfigService configService;
    @Autowired
    private OpHostService opHostService;
    @Autowired
    private UserSilentService userSilentService;
    @Autowired
    private RedisService redisService;

    /**
     * 接收微信静默授权，获取用户信息，跳转前台页面
     *
     * @param appid
     * @param code
     * @param map
     * @return
     */
    @RequestMapping("/login")

    public String login(@RequestParam String appid, @RequestParam String code, @RequestParam String state, ModelMap map) {

        String[] s = state.split("_");
        if (s.length != 2) {
//            logger.error("静默登录state错误");
//            String frontUrl = getFrontUrl(ophost, page, userAdminId, userAdminId);
            return "error";
        }
        String page = s[0];
        Integer channelId = Integer.parseInt(s[1]);

        //来源
        AdminConfig fromAdminConfig = adminConfigService.selectByAppid(appid);
        Integer fromPlatformId = fromAdminConfig.getPlatformId();

        //自身
        AdminConfig adminConfig = adminConfigService.selectByAdminId(channelId);
        Integer ophostId = adminConfig.getOphostId();
        Ophost ophost = opHostService.selectById(ophostId);
        Integer userAdminId = adminConfig.getAdminId();


        Config config = configService.selectByName(SiteCons.SILENT_DEFAULT_CHANNEL);
        Integer defaultAdminId = Integer.parseInt(config.getValue());
        AdminConfig defaultAdminConfig = adminConfigService.selectByAdminId(defaultAdminId);

        WxOpenService wxOpenService = thirdPartService.getWxOpenServiceByPlatFormId(fromPlatformId);
        try {
            WxOAuth2AccessToken accessToken = wxOpenService.getWxOpenComponentService()
                    .getWxMpServiceByAppid(appid).getOAuth2Service().getAccessToken(code);
            String openId = accessToken.getOpenId();
            User user = userService.selectByOpenId(openId);
            if (user == null) {


                //todo 获取基本信息,save
                WxOpenMpService wxMpServiceByAppid = wxOpenService.getWxOpenComponentService().getWxMpServiceByAppid(appid);
             /*   WxMpUser wxMpUser = wxMpServiceByAppid.getUserService()
                        .userInfo(openId, null);*/
                WxOAuth2UserInfo userInfo = wxMpServiceByAppid.getOAuth2Service().getUserInfo(accessToken, "");
                user = userService.createUserByWxOAuth2UserInfo(userInfo, channelId);
                user = userService.selectByOpenId(userInfo.getOpenid());

            }
            boolean twoLogin = false;
            if (twoLogin){

                return twologin(appid, page, channelId, adminConfig, ophost, userAdminId, defaultAdminId, defaultAdminConfig, openId, user);
            }else {
                String frontUrl = getFrontUrl(adminConfig.getAppid(), ophost, page, userAdminId, String.valueOf(user.getId()));
                log.info("跳转：" + frontUrl);
                return "redirect:" + frontUrl;
            }
        } catch (WxErrorException e) {
//            logger.error(e.getMessage());
            e.printStackTrace();
            String frontUrl = getFrontUrl(adminConfig.getAppid(), ophost, state, 0, "");
            return "redirect:" + frontUrl;
        }


    }

    private String twologin(String appid, String page, Integer channelId, AdminConfig adminConfig, Ophost ophost, Integer userAdminId, Integer defaultAdminId, AdminConfig defaultAdminConfig, String openId, User user) {
        WxOpenService wxOpenService;
        if (userSlientExist(user, null, defaultAdminConfig.getAppid())) {
            String frontUrl = getFrontUrl(adminConfig.getAppid(), ophost, page, userAdminId, String.valueOf(user.getId()));
            log.info("跳转：" + frontUrl);
            return "redirect:" + frontUrl;
        } else {
            if (defaultAdminId.equals(channelId)) {
                //保存静默授权
                saveUserSlitent(user, openId, appid);
                String frontUrl = getFrontUrl(adminConfig.getAppid(), ophost, page, userAdminId, String.valueOf(user.getId()));
                log.info("跳转：" + frontUrl);
                return "redirect:" + frontUrl;
            } else {
                Integer platformId = defaultAdminConfig.getPlatformId();
                Ophost menuHost = opHostService.selectById(defaultAdminConfig.getOphostId());
                String redirect_url = String.format(UrlCons.OAUTH2_REDIRECT_URL_SLIENT, menuHost.getHost());

                wxOpenService = thirdPartService.getWxOpenServiceByPlatFormId(platformId);
                String auto2Url = wxOpenService.getWxOpenComponentService().getWxMpServiceByAppid(defaultAdminConfig.getAppid()).getOAuth2Service()
                        .buildAuthorizationUrl(redirect_url, WxConsts.OAuth2Scope.SNSAPI_BASE, page + "_" + user.getId());
                log.info("跳转：" + auto2Url);
                return "redirect:" + auto2Url;
            }


        }
    }

    @RequestMapping("/slient")
    public String slient(@PathVariable String appid, @RequestParam String code, @RequestParam String state, ModelMap map) {
        String[] s = state.split("_");
        if (s.length != 3) {
//            logger.error("静默登录state错误");
//            String frontUrl = getFrontUrl(ophost, "page", userAdminId, userAdminId);
            return "error";
        }
        String page = s[0];
        Long userId = Long.parseLong(s[1]);

        User user = userService.selectById(userId);
        //来源
        AdminConfig fromAdminConfig = adminConfigService.selectByAppid(appid);
        Integer fromPlatformId = fromAdminConfig.getPlatformId();

        //自身
        AdminConfig adminConfig = adminConfigService.selectByAdminId(user.getChannelId());
        Integer ophostId = adminConfig.getOphostId();
        Ophost ophost = opHostService.selectById(ophostId);
        Integer userAdminId = adminConfig.getAdminId();


        WxOpenService wxOpenService = thirdPartService.getWxOpenServiceByPlatFormId(fromPlatformId);
        try {
            WxOAuth2AccessToken accessToken = wxOpenService.getWxOpenComponentService().getWxMpServiceByAppid(appid).getOAuth2Service().getAccessToken(code);
            String openId = accessToken.getOpenId();
            saveUserSlitent(user, openId, appid);
            String frontUrl = getFrontUrl(adminConfig.getAppid(), ophost, page, userAdminId, String.valueOf(user.getId()));
            return "redirect:" + frontUrl;
        } catch (Exception e) {
            e.printStackTrace();
            String frontUrl = getFrontUrl(adminConfig.getAppid(), ophost, page, 0, "");
            return "redirect:" + frontUrl;
        }


    }

    private void saveUserSlitent(User user, String openId, String appid) {
        log.debug(JsonUtils.toJsonStr(user));
        UserSilent userSilent = new UserSilent();
        userSilent.setUserId(user.getId());
        userSilent.setChannelId(user.getChannelId());
        userSilent.setOpenid(user.getOpenid());
        userSilent.setTriAppid(appid);
        userSilent.setTriOpenid(openId);
        log.debug(JsonUtils.toJsonStr(userSilent));
        userSilentService.insert(userSilent);
    }

    private boolean userSlientExist(User user, String openId, String appid) {
        UserSilent userSilent = new UserSilent();
        userSilent.setUserId(user.getId());
        userSilent.setChannelId(user.getChannelId());
        userSilent.setOpenid(user.getOpenid());
        userSilent.setTriAppid(appid);
        userSilent.setTriOpenid(openId);
        log.debug(JsonUtils.toJsonStr(userSilent));
        UserSilent result = userSilentService.selectByUserSlient(userSilent);
        return result != null;


    }

    private String getFrontUrl(String appid, Ophost ophost, String state, Integer userAdminId, String userId) {

        String s = redisService.get(state);

        if (s != null && s.startsWith("http")) {
            String sp = s.contains("?") ? "&" : "?";
            s = String.format(s + sp + UrlCons.URL_PARAMS, userAdminId, userId);
            return s;
        } else {
            String host = ophost.getHost();
            String frontUrl = String.format(UrlCons.FRONT_URL, appid, host, state, userAdminId, userId);
            return frontUrl;
        }

    }
}
