package com.book.push.service.push;

import java.util.concurrent.Delayed;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.atomic.AtomicLong;

/**
 * Delayed接口的实现类
 * 内部实现了getDelay()和compareTo()方法,分别用来获取延迟时间和按两个任务的延迟时间进行排序
 * * * @author lijilei
 */
public abstract class DelayTask<T> implements Delayed, Runnable {
    private static final long NANO_ORIGIN = System.nanoTime();

    final static long now() {
        return System.nanoTime() - NANO_ORIGIN;
    }

    private static final AtomicLong sequencer = new AtomicLong(0);
    private final long sequenceNumber;
    private final long time;
    protected final T entity;

    public DelayTask(T entity, long timeout) {
        this.time = now() + timeout;
        this.entity = entity;
        this.sequenceNumber = sequencer.getAndIncrement();
    }

    public T getEntity() {
        return this.entity;
    }

    @Override
    public long getDelay(TimeUnit unit) {
        long d = unit.convert(time - now(), TimeUnit.NANOSECONDS);
        return d;
    }

    @Override
    public int compareTo(Delayed other) {
        if (other == this) {
            return 0;
        }
        if (other instanceof DelayTask) {
            DelayTask<?> x = (DelayTask<?>) other;
            long diff = time - x.time;
            if (diff < 0) return -1;
            else if (diff > 0) return 1;
            else if (sequenceNumber < x.sequenceNumber) return -1;
            else return 1;
        }
        long d = (getDelay(TimeUnit.NANOSECONDS) - other.getDelay(TimeUnit.NANOSECONDS));
        return (d == 0) ? 0 : ((d < 0) ? -1 : 1);
    }

    @Override
    public boolean equals(Object obj) {
        return super.equals(obj);
    }
}