package com.book.push.service.push.impl;

import com.book.dao.cps.pojo.*;
import com.book.dao.cpsshard.entity.Sign;
import com.book.dao.cpsshard.entity.UserRecentlyRead;
import com.book.dao.cpsshard.mapper.SignMapper;
import com.book.dao.cpsshard.pojo.example.SignExample;
import com.book.dao.utils.TimeUtil;
import com.book.push.service.dao.*;
import com.book.push.service.push.IntellectPushService;
import com.book.push.service.redis.RedisService;
import com.book.push.service.wx.WxThirdPartService;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.error.WxErrorException;
import me.chanjar.weixin.mp.bean.kefu.WxMpKefuMessage;
import me.chanjar.weixin.mp.builder.kefu.NewsBuilder;
import me.chanjar.weixin.mp.builder.kefu.TextBuilder;
import me.chanjar.weixin.open.api.WxOpenMpService;
import me.chanjar.weixin.open.api.WxOpenService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;

import java.util.Calendar;

/**
 * created in 2021/9/7
 * Project: book-store
 *
 * @author win7
 */
@Service
@Slf4j
public class IntellectPushServiceImpl implements IntellectPushService {
    @Autowired
    private WechatAutoreplayService wechatAutoreplayService;
    @Autowired
    private AdminConfigService adminConfigService;
    @Autowired
    private UserService userService;
    @Autowired
    private OpHostService opHostService;
    @Autowired
    private SignMapper signMapper;
    @Autowired
    private SignRecommendService signRecommendService;
    @Autowired
    private SignedRecommendService signedRecommendService;
    @Autowired
    private ManageTitleService manageTitleService;
    @Autowired
    private ManageCoverService manageCoverService;
    @Autowired
    private OrdersService ordersService;
    @Autowired
    private ConfigService configService;
    @Autowired
    private BookService bookService;
    @Autowired
    private RedisService redisService;
    @Lazy
    @Autowired
    private WxThirdPartService wxThirdPartService;
    @Autowired
    private UserRecentReadService userRecentReadService;

    public WxMpKefuMessage createSubscribeMessage(Integer platformId, String appid, Long userId) {
        return createSignMessage(platformId, appid, userId);
    }

    public WxMpKefuMessage createSignMessage(Integer platformId, String appid, Long userId) {
        Long bookId;
        User user = userService.selectById(userId);
        AdminConfig adminConfig = adminConfigService.selectByAppid(appid);
        SignExample example = SignExample.newAndCreateCriteria().andUidEqualTo(user.getId().intValue()).andCreatedateEqualTo(TimeUtil.getCurrentIntDate(TimeUtil.YYYYMMDD_N)).example();
        Sign sign = signMapper.selectOneByExample(example);

        //修改，关闭未签到推荐书库
//        if (sign == null) {
        SignedRecommand signRecommand = signedRecommendService.selectRandom(user.getSex().toString());
        bookId = signRecommand.getBookId();
     /*   } else {
            SignRecommand signRecommand = signRecommendService.selectRandom(user.getSex().toString());
            bookId = signRecommand.getBookId();

        }*/
        ManageTitle manageTitle = manageTitleService.selectRandom(user.getSex().toString());
        ManageCover manageCover = manageCoverService.selectRandom("1", user.getSex().toString());
        WxMpKefuMessage.WxArticle wxArticle = new WxMpKefuMessage.WxArticle();
        wxArticle.setTitle(manageTitle.getTitle());
        wxArticle.setPicUrl(manageCover.getImage());
        wxArticle.setUrl(bookService.createBookUrl(bookId, adminConfig, user));
        WxMpKefuMessage wxMpKefuMessage = new NewsBuilder().toUser(user.getOpenid()).addArticle(wxArticle).build();
        return wxMpKefuMessage;
    }

    public WxMpKefuMessage createContinueReadMessage(Integer platformId, String appid, Integer id) {
        UserRecentlyRead userRecentlyRead = userRecentReadService.selectUserRecentlyReadById(id);
        Long userId = userRecentlyRead.getUserId();
        User user = userService.selectById(userId);
        Integer channelId = user.getChannelId();
        AdminConfig adminConfig = adminConfigService.selectByAdminId(channelId);
        Book book = bookService.selectById(userRecentlyRead.getBookId());
        WxOpenService wxOpenService = wxThirdPartService.getWxOpenServiceByPlatFormId(platformId);
        WxOpenMpService wxMpService = wxOpenService.getWxOpenComponentService().getWxMpServiceByAppid(appid);
        String msg = "亲，你上次看的书还在等着你呐~ 点击继续阅读%n>%s%n首冲50元即可畅读90万字，365元年份会员更可全年无线畅读。%n%n为了下次方便阅读，请置顶公众号";
        Long bookId = userRecentlyRead.getBookId();
        String url = bookService.createBookUrl(bookId, adminConfig, user);
        String link = "<a href='" + url + "'>《" + book.getName() + "》</a>";
        msg = String.format(msg, link);
        WxMpKefuMessage wxMpKefuMessage = new TextBuilder().toUser(user.getOpenid()).content(msg).build();


        return wxMpKefuMessage;
    }

    public WxMpKefuMessage createNopayMessage(Integer platformId, String appid, Integer id) {
        Orders orders = ordersService.selectById(id);
        User user = userService.selectById(orders.getUserId());
        String msg = "亲，你上次看的书还在等着你呐~ 点击继续阅读%n>%s%n首冲50元即可畅读90万字，365元年份会员更可全年无线畅读。%n%n为了下次方便阅读，请置顶公众号";
        Long bookId = orders.getBookId();
        if (bookId == null) {

            UserRecentlyRead userRecentlyRead = userRecentReadService.selectLastByUserId(orders.getUserId());
            bookId = userRecentlyRead.getBookId();
        }
        AdminConfig adminConfig = adminConfigService.selectByAppid(appid);

        String url = bookService.createBookUrl(bookId, adminConfig, user);
        Book book = bookService.selectById(bookId);
        String link = "<a href='" + url + "'>《" + book.getName() + "》</a>";
        msg = String.format(msg, link);
        WxMpKefuMessage wxMpKefuMessage = new TextBuilder().toUser(user.getOpenid()).content(msg).build();
        return wxMpKefuMessage;
    }

    public WxMpKefuMessage createNoSignMessage(Integer platformId, String appid, Long userId) {
        String msg = String.format("亲，您今天还没有签到哦，点击%n签到即可领取书币哟~%n%n<a href='weixin://bizmsgmenu?msgmenucontent=签到&msgmenuid=0'>点我签到</a>%n%n为了方便下次阅读，请置顶公众号");
        User user = userService.selectById(userId);
        WxMpKefuMessage wxMpKefuMessage = new TextBuilder().toUser(user.getOpenid()).content(msg).build();
        return wxMpKefuMessage;
    }

    @Override
    public void pushBySubscribe(Integer platformId, String appid, Long userId) {
        try {
            Integer second = afterNow();
            if (second > 0) {
                redisService.set(redisService.getKeyListener().getKeyMpSubscribe(platformId, appid, userId), userId + "", second);
                return;
            }
            WxMpKefuMessage signMessage = createSubscribeMessage(platformId, appid, userId);


            WxOpenService wxOpenService = wxThirdPartService.getWxOpenServiceByPlatFormId(platformId);
            WxOpenMpService wxMpService = wxOpenService.getWxOpenComponentService().getWxMpServiceByAppid(appid);

            wxMpService.getKefuService().sendKefuMessage(signMessage);
        } catch (WxErrorException e) {
            e.printStackTrace();
            log.error("签到智能推送失败", e);
        }

    }

    @Override
    public void pushBySign(Integer platformId, String appid, Long userId) {
        try {
            Integer second = afterNow();
            if (second > 0) {
                redisService.set(redisService.getKeyListener().getKeyMpSign(platformId, appid, userId), userId + "", second);
                return;
            }
            WxMpKefuMessage signMessage = createSignMessage(platformId, appid, userId);


            WxOpenService wxOpenService = wxThirdPartService.getWxOpenServiceByPlatFormId(platformId);
            WxOpenMpService wxMpService = wxOpenService.getWxOpenComponentService().getWxMpServiceByAppid(appid);

            wxMpService.getKefuService().sendKefuMessage(signMessage);
        } catch (WxErrorException e) {
            e.printStackTrace();
            log.error("签到智能推送失败", e);
        }


    }


    @Override
    public void pushByContinueRead(Integer platformId, String appid, Integer id) {
        try {
            Integer second = afterNow();
            if (second > 0) {
                redisService.set(redisService.getKeyListener().getKeyPrefixBookContinueRead(platformId, appid, id), id + "", second);
                return;
            }
            WxMpKefuMessage signMessage = createContinueReadMessage(platformId, appid, id);


            WxOpenService wxOpenService = wxThirdPartService.getWxOpenServiceByPlatFormId(platformId);
            WxOpenMpService wxMpService = wxOpenService.getWxOpenComponentService().getWxMpServiceByAppid(appid);

            wxMpService.getKefuService().sendKefuMessage(signMessage);
        } catch (WxErrorException e) {
            e.printStackTrace();
            log.error("继续阅读智能推送失败", e);
        }

    }

    @Override
    public void pushByNopay(Integer platformId, String appid, Integer id) {
        try {
            WxMpKefuMessage signMessage = createNopayMessage(platformId, appid, id);

            WxOpenService wxOpenService = wxThirdPartService.getWxOpenServiceByPlatFormId(platformId);
            WxOpenMpService wxMpService = wxOpenService.getWxOpenComponentService().getWxMpServiceByAppid(appid);

            wxMpService.getKefuService().sendKefuMessage(signMessage);
        } catch (WxErrorException e) {
            e.printStackTrace();
            log.error("未支付提醒智能推送失败", e);
        }
    }

    @Override
    public void pushByNoSign(Integer platformId, String appid, Long userId) {
        try {
            Integer second = afterNow();
            if (second > 0) {
                redisService.set(redisService.getKeyListener().getKeyMpNosign(platformId, appid, userId), userId + "", second);
                return;
            }
            WxMpKefuMessage signMessage = createNoSignMessage(platformId, appid, userId);

            WxOpenService wxOpenService = wxThirdPartService.getWxOpenServiceByPlatFormId(platformId);
            WxOpenMpService wxMpService = wxOpenService.getWxOpenComponentService().getWxMpServiceByAppid(appid);

            wxMpService.getKefuService().sendKefuMessage(signMessage);
        } catch (WxErrorException e) {
            e.printStackTrace();
            log.error("未签到提醒智能推送失败", e);
        }
    }


    private Integer afterNow() {
        Calendar calendar = Calendar.getInstance();
        int i = calendar.get(Calendar.HOUR_OF_DAY);
        return (6 - i) * 3600;
    }
}
