package com.book.push.service.push.impl;

import com.book.dao.cps.pojo.Custom;
import com.book.push.service.dao.CustomService;
import com.book.push.service.push.DelayTask;
import com.book.push.service.push.vo.CustomMessage;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.error.WxErrorException;
import me.chanjar.weixin.mp.bean.kefu.WxMpKefuMessage;
import me.chanjar.weixin.open.api.WxOpenMpService;

import java.util.List;

/**
 * created in 2021/8/29
 * Project: book-store
 *
 * @author win7
 */
@Slf4j
public class PushCustomMessageTask extends DelayTask<CustomMessage> {
    private WxOpenMpService wxMpService;

    public PushCustomMessageTask(CustomMessage entity, long timeout) {
        super(entity, timeout);
    }

    public PushCustomMessageTask(WxOpenMpService wxMpService, CustomMessage entity, long timeout) {

        super(entity, timeout);
        this.wxMpService = wxMpService;
    }

    @Override
    public void run() {
        List<WxMpKefuMessage> messageList = entity.getMessageList();
        log.info("发送客服消息：共{}人，消息内容：\n{}",messageList.size(),messageList);
        int success = 0;
        int fail = 0;
        for (int i = 0; i < messageList.size(); i++) {
            WxMpKefuMessage wxMpKefuMessage = messageList.get(i);
            try {
                wxMpService.getKefuService().sendKefuMessage(wxMpKefuMessage);
                success++;
            } catch (WxErrorException e) {
                fail++;
                System.out.println("客服笑死发送失败：" + wxMpKefuMessage);
                log.warn("客服消息发送失败：" + wxMpKefuMessage);
                e.printStackTrace();
            }
        }
        log.info("发送成功：" + success + ",失败：:" + fail);
        Custom custom = entity.getCustom();
        custom.setSuccessNum(success);
        custom.setFailNum(fail);
        CustomService customService = entity.getCustomService();
        customService.update(custom);

    }
}