package com.book.push.service.push.impl;

import com.book.push.service.push.DelayTask;
import com.book.push.service.push.PushService;
import com.book.push.service.push.vo.BookInfo;
import com.book.push.service.push.vo.CustomMessage;
import me.chanjar.weixin.mp.bean.kefu.WxMpKefuMessage;
import me.chanjar.weixin.open.api.WxOpenMpService;
import org.springframework.stereotype.Service;

import java.util.concurrent.*;

/**
 * created in 2021/8/27
 * Project: book-store
 *
 * @author win7
 */
@Service
public  class PushServiceImpl implements PushService {

    /**
     * 核心线程数
     */
    private final static int CORE_POOL_SIZE = 8;
    /**
     * 最大线程数
     */
    private final static int MAX_POOL_SIZE = 128;
    /**
     * 缓存队列容量
     */
    private final static int QueueSize = 128;

    /**
     * 线程空闲存活时间
     */
    private final static int KEEP_ALIVE_TIME = 60;
    /**
     * 缓存队列
     */
    private final static BlockingQueue<Runnable> teskQueue = new LinkedBlockingQueue<Runnable>(QueueSize);
    /**
     * 线程池
     */
    private final static ThreadPoolExecutor THREAD_POOL_EXECUTOR;

    static {

        ThreadPoolExecutor threadPoolExecutor = new ThreadPoolExecutor(CORE_POOL_SIZE, MAX_POOL_SIZE, KEEP_ALIVE_TIME, TimeUnit.SECONDS, teskQueue);
        THREAD_POOL_EXECUTOR = threadPoolExecutor;

    }
    private static final DelayQueue<DelayTask> queue = new DelayQueue<>();

    private final Thread taskThread;

    private PushServiceImpl() {

        taskThread = new Thread(new Runnable() {
            @Override
            public void run() {
                execute();
            }
        });
        taskThread.setName("NotifyService Thread");
        taskThread.start();
    }

    private void execute() {
        for (; ; ) {
            try {
                DelayTask delayItem = queue.take();
                if (delayItem != null) {
                    THREAD_POOL_EXECUTOR.execute(delayItem);
                }
            } catch (InterruptedException e) {
                e.printStackTrace();

                continue;
            }
        }
    }

    @Override
    public void addPushTask(DelayTask task) {
        queue.put(task);
    }

    @Override
    public void addBookPushTask(BookInfo bookInfo, long delay) {


        queue.put(new PushBookTask(bookInfo,delay));

    }

    @Override
    public void addWxKefuMessagePushTask(WxOpenMpService wxMpService, WxMpKefuMessage wxMpKefuMessage, long delay) {
        queue.put(new PushMessageTask(wxMpService,wxMpKefuMessage,delay));
    }
    @Override
    public void addWxKefuMessagePushTask(WxOpenMpService wxMpService, WxMpKefuMessage wxMpKefuMessage, long delay, TimeUnit timeUnit) {
        long nanoTime =  TimeUnit.NANOSECONDS.convert(delay, timeUnit);
        queue.put(new PushMessageTask(wxMpService,wxMpKefuMessage,nanoTime));
    }

    @Override
    public void addCustomMessagePushTask(WxOpenMpService wxMpService, CustomMessage customMessage, long delay, TimeUnit timeUnit) {
        long nanoTime =  TimeUnit.NANOSECONDS.convert(delay, timeUnit);
        queue.put(new PushCustomMessageTask(wxMpService,customMessage,nanoTime));
    }
}
