package com.book.push.service.redis.impl;

import com.book.push.config.KeyExpiredListener;
import com.book.push.service.redis.RedisService;
import lombok.Cleanup;
import lombok.Getter;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import redis.clients.jedis.Jedis;
import redis.clients.jedis.JedisPool;

import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.TimeUnit;

/**
 * created in 2021/8/20
 * Project: book-store
 *
 * @author win7
 */
@Service
public class RedisServiceImpl implements RedisService {
    @Autowired
    private KeyExpiredListener keyExpiredListener;

    @Autowired
    private JedisPool jedisPool;


    @Override
    public void set(String key, String value) {
        @Cleanup Jedis jedis = jedisPool.getResource();
        jedis.set(key, value);
    }

    @Override
    public String get(String key) {
        @Cleanup Jedis jedis = jedisPool.getResource();
        return jedis.get(key);
    }

    @Override
    public Long getExpiresTime(String key) {
        @Cleanup Jedis jedis = jedisPool.getResource();
        return jedis.ttl(key);

    }

    @Override
    public void setMap(String key, Map<String, String> map) {
        @Cleanup Jedis jedis = jedisPool.getResource();
        jedis.hmset(key, map);
    }

    @Override
    public List<String> getByMapKey(String key, String mapKey) {
        @Cleanup Jedis jedis = jedisPool.getResource();
        return jedis.hmget(key, mapKey);
    }

    @Override
    public void setList(String key, String... val) {
        @Cleanup Jedis jedis = jedisPool.getResource();
        for (String s : val) {

            jedis.lpush(key, s);
        }
    }

    @Override
    public void setList(String key, List<String> list) {
        @Cleanup Jedis jedis = jedisPool.getResource();
        for (String s : list) {

            jedis.lpush(key, s);
        }
    }

    @Override
    public List<String> getList(String key, int begin, int end) {
        @Cleanup Jedis jedis = jedisPool.getResource();
        return jedis.lrange(key, begin, end);
    }

    @Override
    public void setSet(String key, Set<String> set) {
        @Cleanup Jedis jedis = jedisPool.getResource();
        for (String s : set) {

            jedis.sadd(key, s);
        }
    }

    @Override
    public Set<String> getSet(String key) {
        @Cleanup Jedis jedis = jedisPool.getResource();
        return jedis.smembers(key);
    }

    @Override
    public void set(String key, String  value, long second) {
        @Cleanup Jedis jedis = jedisPool.getResource();
        jedis.psetex(key, TimeUnit.SECONDS.toMillis(second), value);
    }

    @Override
    public Long del(String key){
        @Cleanup Jedis jedis = jedisPool.getResource();
      return jedis.del(key);
    }

    @Override
    public KeyExpiredListener getKeyListener() {
        return keyExpiredListener;
    }
}
