package com.book.push.service.wx.impl;

import com.book.dao.cps.pojo.AdminConfig;
import com.book.dao.cps.pojo.Platform;
import com.book.push.config.RedisProperties;
import com.book.push.config.WxOpenInRedisBookConfigStorage;
import com.book.push.handler.*;
import com.book.push.service.dao.AdminConfigService;
import com.book.push.service.dao.PlatformService;
import com.book.push.service.redis.RedisService;
import com.book.push.service.wx.WxThirdPartService;
import com.book.push.utils.JsonUtils;
import com.book.push.utils.Md5Util;
import com.book.push.vo.AuthorizerInfo;
import com.book.push.vo.ComponentInfo;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import me.chanjar.weixin.common.bean.oauth2.WxOAuth2AccessToken;
import me.chanjar.weixin.common.error.WxRuntimeException;
import me.chanjar.weixin.open.api.WxOpenConfigStorage;
import me.chanjar.weixin.open.api.WxOpenService;
import me.chanjar.weixin.open.api.impl.WxOpenServiceImpl;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.stereotype.Service;
import redis.clients.jedis.JedisPool;

import javax.annotation.PostConstruct;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.Lock;

import static me.chanjar.weixin.common.api.WxConsts.EventType;
import static me.chanjar.weixin.common.api.WxConsts.EventType.SUBSCRIBE;
import static me.chanjar.weixin.common.api.WxConsts.EventType.UNSUBSCRIBE;
import static me.chanjar.weixin.common.api.WxConsts.XmlMsgType;
import static me.chanjar.weixin.common.api.WxConsts.XmlMsgType.EVENT;
import static me.chanjar.weixin.mp.constant.WxMpEventConstants.CustomerService.*;
import static me.chanjar.weixin.mp.constant.WxMpEventConstants.POI_CHECK_NOTIFY;

/**
 * created in 2021/8/20
 * Project: book-store
 *
 * @author win7
 */
@Service
@Slf4j
@RequiredArgsConstructor
@EnableConfigurationProperties(RedisProperties.class)
public class WxThirdPartServiceImpl implements WxThirdPartService {

    private final LogHandler logHandler;
    private final NullHandler nullHandler;
    private final KfSessionHandler kfSessionHandler;
    private final StoreCheckNotifyHandler storeCheckNotifyHandler;
    private final LocationHandler locationHandler;
    private final MenuHandler menuHandler;
    private final MsgHandler msgHandler;
    private final UnsubscribeHandler unsubscribeHandler;
    private final SubscribeHandler subscribeHandler;
    private final ScanHandler scanHandler;
    private volatile ConcurrentHashMap<Integer, WxOpenService> wxOpenServiceMap = new ConcurrentHashMap();
    @Autowired
    private RedisProperties redisProperties;
    @Autowired
    private PlatformService platformService;
    @Autowired
    private RedisService redisService;
    @Autowired
    private JedisPool jedisPool;
    @Autowired
    private AdminConfigService adminConfigService;

    private WxThirdPartMessageRouter wxThirdPartMessageRouter;


    @PostConstruct
    public void init() {
        log.info("------------------------------                       WxThirdPartServiceImpl  init                       ----------------------------------");
        Map<Integer, Platform> integerPlatformMap = platformService.selectAll();
        Set<Integer> platFormIdSet = integerPlatformMap.keySet();
        platFormIdSet.stream().forEach(id -> {
            Platform platform = integerPlatformMap.get(id);
            log.info("platform_id:" + id + "-->" + platform.toString());
            WxOpenService wxOpenService = new WxOpenServiceImpl();
            WxOpenInRedisBookConfigStorage inRedisConfigStorage = new WxOpenInRedisBookConfigStorage(platform, jedisPool);
            inRedisConfigStorage.setComponentAppId(platform.getAppid());
            inRedisConfigStorage.setComponentAppSecret(platform.getSecret());
            inRedisConfigStorage.setComponentAesKey(platform.getAesKey());
            inRedisConfigStorage.setComponentToken(platform.getToken());
            inRedisConfigStorage.setRedisService(redisService);
//            inRedisConfigStorage.setAdminConfigList(adminConfigService.selectByPlatformId(platform.getId()));
            inRedisConfigStorage.setAdminConfigService(adminConfigService);

            inRedisConfigStorage.updateComponentVerifyTicketFromPhp();
            inRedisConfigStorage.updateComponentAccessTokenFromPhp();
            inRedisConfigStorage.updateAllAuthorizerAccessToken();

            wxOpenService.setWxOpenConfigStorage(inRedisConfigStorage);

            wxOpenServiceMap.put(id, wxOpenService);
        });


        wxThirdPartMessageRouter = new WxThirdPartMessageRouter(this);
        // 记录所有事件的日志 （异步执行）
        wxThirdPartMessageRouter.rule().handler(this.logHandler).next();

        // 接收客服会话管理事件
        wxThirdPartMessageRouter.rule().async(false).msgType(EVENT).event(KF_CREATE_SESSION)
                .handler(this.kfSessionHandler).end();
        wxThirdPartMessageRouter.rule().async(false).msgType(EVENT).event(KF_CLOSE_SESSION)
                .handler(this.kfSessionHandler).end();
        wxThirdPartMessageRouter.rule().async(false).msgType(EVENT).event(KF_SWITCH_SESSION)
                .handler(this.kfSessionHandler).end();

        // 门店审核事件
        wxThirdPartMessageRouter.rule().async(false).msgType(EVENT).event(POI_CHECK_NOTIFY).handler(this.storeCheckNotifyHandler).end();

        // 自定义菜单事件
        wxThirdPartMessageRouter.rule().async(false).msgType(EVENT).event(EventType.CLICK).handler(this.menuHandler).end();

        // 点击菜单连接事件
        wxThirdPartMessageRouter.rule().async(false).msgType(EVENT).event(EventType.VIEW).handler(this.nullHandler).end();

        // 关注事件
        wxThirdPartMessageRouter.rule().async(false).msgType(EVENT).event(SUBSCRIBE).handler(this.subscribeHandler).end();

        // 取消关注事件
        wxThirdPartMessageRouter.rule().async(false).msgType(EVENT).event(UNSUBSCRIBE).handler(this.unsubscribeHandler).end();

        // 上报地理位置事件
        wxThirdPartMessageRouter.rule().async(false).msgType(EVENT).event(EventType.LOCATION).handler(this.locationHandler).end();

        // 接收地理位置消息
        wxThirdPartMessageRouter.rule().async(false).msgType(XmlMsgType.LOCATION).handler(this.locationHandler).end();

        // 扫码事件
        wxThirdPartMessageRouter.rule().async(false).msgType(EVENT).event(EventType.SCAN).handler(this.scanHandler).end();

        // 默认
        wxThirdPartMessageRouter.rule().async(false).handler(this.msgHandler).end();


    }


    @Override
    public WxOpenService getWxOpenServiceByPlatFormId(Integer platFormId) {
        return wxOpenServiceMap.get(platFormId);
    }

    @Override
    public WxThirdPartMessageRouter getWxThirdPartMessageRouter() {
        return this.wxThirdPartMessageRouter;
    }

    @Override
    public WxOpenService getWxOpenServiceByAuthorizerAppid(String appid) {
        AdminConfig adminConfig = adminConfigService.selectByAppid(appid);


        Integer platformId = adminConfig.getPlatformId();
        WxOpenService wxOpenService = getWxOpenServiceByPlatFormId(platformId);
        return wxOpenService;
    }

    public static void main(String[] args) {
        ComponentInfo componentInfo = new ComponentInfo("wxa6f2133192bfbcac", "a828e6b5eb87419e50826c60b9a20201"
                , "ticket@@@qP9Pd4w50ic2Jws07j4a9qbc0YXLwWi8YLjcII08Rc0rSC0djpv_fD20AGqWlTSXgyFzmyWuPgBeNm8o8ImzfQ");
        System.out.println(JsonUtils.toJsonString(componentInfo));
        String s = WxThirdPartService.COMPONENT_TOKEN_KEY_PREFIX + Md5Util.md5(JsonUtils.toJsonString(componentInfo));

        System.out.println(s);

        String r = "{\"component_appid\":\"wxa6f2133192bfbcac\",\"component_appsecret\":\"a828e6b5eb87419e50826c60b9a20201\",\"component_verify_ticket\":\"ticket@@@qP9Pd4w50ic2Jws07j4a9qbc0YXLwWi8YLjcII08Rc0rSC0djpv_fD20AGqWlTSXgyFzmyWuPgBeNm8o8ImzfQ\"}";
        System.out.println("real=" + Md5Util.md5(r));

        //
        System.out.println("------------------------------------------------------------------------------------------");


        AuthorizerInfo authorizerInfo = new AuthorizerInfo("wxa6f2133192bfbcac", "wxe1271417b2ff5b1c", "refreshtoken@@@N0INeuo9aWT1PdFNIfSTynUm8eI5gGp9fmg_SAG69p8");
        System.out.println(JsonUtils.toJsonNoSort(authorizerInfo));
        System.out.println("{\"component_appid\":\"wxa6f2133192bfbcac\",\"authorizer_appid\":\"wxe1271417b2ff5b1c\",\"authorizer_refresh_token\":\"refreshtoken@@@N0INeuo9aWT1PdFNIfSTynUm8eI5gGp9fmg_SAG69p8\"}");
        String s1 = WxThirdPartService.COMPONENT_TOKEN_KEY_PREFIX + Md5Util.md5(JsonUtils.toJsonNoSort(authorizerInfo));
        System.out.println(s1);
        System.out.println(Md5Util.md5("{\"component_appid\":\"wxa6f2133192bfbcac\",\"authorizer_appid\":\"wxe1271417b2ff5b1c\",\"authorizer_refresh_token\":\"refreshtoken@@@N0INeuo9aWT1PdFNIfSTynUm8eI5gGp9fmg_SAG69p8\"}"));
    }

    @Override
    public WxOAuth2AccessToken getOAuth2AccessTokenByRefreshToken(Integer platformId, String appid) {
        WxOpenService wxOpenService = getWxOpenServiceByPlatFormId(platformId);
        WxOpenInRedisBookConfigStorage wxOpenConfigStorage = (WxOpenInRedisBookConfigStorage) wxOpenService.getWxOpenConfigStorage();

        if (wxOpenConfigStorage.isOAuth2rRefreshTokenExpired(appid)) {
            return null;
        }
        String oAuth2rRefreshToken = wxOpenConfigStorage.getOAuth2rRefreshToken(appid);
        try {
            log.info("查询oauth2 refresh token:" + oAuth2rRefreshToken);
            return wxOpenService.getWxOpenComponentService().oauth2refreshAccessToken(appid,oAuth2rRefreshToken);
        } catch (Exception e) {
            log.error(e.getMessage());
            e.printStackTrace();
        }
        return null;

    }

    @Override
    public WxOAuth2AccessToken getOAuth2AccessTokenByCode(Integer platformId, String appid, String code) {
        WxOpenService wxOpenService = getWxOpenServiceByPlatFormId(platformId);
        WxOpenInRedisBookConfigStorage wxOpenConfigStorage = (WxOpenInRedisBookConfigStorage) wxOpenService.getWxOpenConfigStorage();
        Lock lock = wxOpenConfigStorage.getOAuth2rRefreshTokenLock(appid);
        boolean locked = false;
        lock.lock();
        try {


            WxOAuth2AccessToken accessToken = wxOpenService.getWxOpenComponentService().getWxMpServiceByAppid(appid).getOAuth2Service().getAccessToken(code);
            wxOpenConfigStorage.updateOAuth2rRefreshToken(appid, accessToken.getRefreshToken(), 60 * 60 * 24 * 30);
            log.info("更新oauth2 refresh token:" + accessToken.getRefreshToken());
            return accessToken;

        } catch (Exception e) {
            throw new WxRuntimeException(e);
        } finally {
            lock.unlock();
        }

    }


}
