package com.book.push.task;

import com.book.dao.cps.pojo.AdminConfig;
import com.book.dao.cps.pojo.Book;
import com.book.dao.cps.pojo.Orders;
import com.book.dao.cps.pojo.User;
import com.book.dao.cpsshard.entity.UserRecentlyRead;
import com.book.dao.utils.DateUtils;
import com.book.push.service.dao.*;
import com.book.push.service.push.PushService;
import com.book.push.service.wx.WxThirdPartService;
import me.chanjar.weixin.mp.bean.kefu.WxMpKefuMessage;
import me.chanjar.weixin.mp.builder.kefu.TextBuilder;
import me.chanjar.weixin.open.api.WxOpenMpService;
import me.chanjar.weixin.open.api.WxOpenService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * created in 2021/9/1
 * Project: book-store
 *
 * @author win7
 */
@Component
public class NoPayTask {
    @Autowired
    private OrdersService ordersService;
    @Autowired
    private AdminConfigService adminConfigService;
    @Autowired
    private WxThirdPartService wxThirdPartService;
    @Autowired
    private UserService userService;
    @Autowired
    private PushService pushService;
    @Autowired
    private UserRecentReadService userRecentReadService;
    @Autowired
    private BookService bookService;

    @Scheduled(fixedRate = 1000 * 60 * 5)
    public void run() {
        Integer from = DateUtils.getNow() - 24 * 60 * 60;
        Integer to = from + (5 * 60);
        List<Orders> orders = ordersService.selectNoPayOrderBetweenCreate(from, to);
        if (orders == null) {
            return;
        }
        for(Orders o:orders) {
            Integer adminId = o.getAdminId();
            Long userId = o.getUserId();
            AdminConfig adminConfig = adminConfigService.selectByAdminId(adminId);
            if (adminConfig.getUnpayTip().equals("0")){
                continue;
            }
            User user = userService.selectById(userId);
            String openid = user.getOpenid();
            String appid = adminConfig.getAppid();
            Integer platformId = adminConfig.getPlatformId();
            WxOpenService wxOpenService = wxThirdPartService.getWxOpenServiceByPlatFormId(platformId);
            WxOpenMpService wxMpService = wxOpenService.getWxOpenComponentService().getWxMpServiceByAppid(appid);
            String msg = "亲，你上次看的书还在等着你呐~ 点击继续阅读%n>%s%n首冲50元即可畅读90万字，365元年份会员更可全年无线畅读。%n%n为了下次方便阅读，请置顶公众号";
            UserRecentlyRead userRecentlyRead = userRecentReadService.selectLastByUserId(userId);
            Long bookId = userRecentlyRead.getBookId();
            String url =bookService.createBookUrl(bookId,adminConfig,user);
            Book book = bookService.selectById(bookId);
            String link = "<a href='"+url+"'>《"+book.getName()+"》</a>";
            msg = String.format(msg,link);
            WxMpKefuMessage wxMpKefuMessage = new TextBuilder().toUser(user.getOpenid()).content(msg).build();
            pushService.addWxKefuMessagePushTask(wxMpService, wxMpKefuMessage, to - (o.getCreatetime()));
        }

    }
}
