package com.book.push.task;

import com.book.dao.cps.pojo.AdminConfig;
import com.book.dao.cps.pojo.Book;
import com.book.dao.cps.pojo.User;
import com.book.dao.cpsshard.entity.UserRecentlyRead;
import com.book.dao.utils.DateUtils;
import com.book.push.service.dao.AdminConfigService;
import com.book.push.service.dao.BookService;
import com.book.push.service.dao.UserRecentReadService;
import com.book.push.service.dao.UserService;
import com.book.push.service.push.PushService;
import com.book.push.service.wx.WxThirdPartService;
import me.chanjar.weixin.mp.bean.kefu.WxMpKefuMessage;
import me.chanjar.weixin.mp.builder.kefu.TextBuilder;
import me.chanjar.weixin.open.api.WxOpenMpService;
import me.chanjar.weixin.open.api.WxOpenService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * created in 2021/9/1
 * Project: book-store
 *
 * @author win7
 */
@Component

public class RecentReadTask {
    @Autowired
    private AdminConfigService adminConfigService;
    @Autowired
    private WxThirdPartService wxThirdPartService;
    @Autowired
    private UserService userService;

    @Autowired
    private UserRecentReadService userRecentReadService;
    @Autowired
    private PushService pushService;
    @Autowired
    private BookService bookService;

    @Scheduled(fixedRate = 1000 * 60 * 4)
    public void run() {
        Integer from = DateUtils.getNow() - 7 * 60 * 60;
        Integer to = from + (4 * 60);
        List<UserRecentlyRead> list = userRecentReadService.selectLastGroupByUserId(from,to);
        for (UserRecentlyRead userRecentlyRead : list) {
            Long userId = userRecentlyRead.getUserId();
            User user = userService.selectById(userId);
            Integer channelId = user.getChannelId();
            AdminConfig adminConfig = adminConfigService.selectByAdminId(channelId);
            Integer platformId = adminConfig.getPlatformId();
            String appid = adminConfig.getAppid();
            Book book = bookService.selectById(userRecentlyRead.getBookId());
            WxOpenService wxOpenService = wxThirdPartService.getWxOpenServiceByPlatFormId(platformId);
            WxOpenMpService wxMpService = wxOpenService.getWxOpenComponentService().getWxMpServiceByAppid(appid);
            String msg = "亲，你上次看的书还在等着你呐~ 点击继续阅读%n>%s%n首冲50元即可畅读90万字，365元年份会员更可全年无线畅读。%n%n为了下次方便阅读，请置顶公众号";
            Long bookId = userRecentlyRead.getBookId();
            String url = bookService.createBookUrl(bookId, adminConfig, user);
            String link = "<a href='" + url + "'>《" + book.getName() + "》</a>";
            msg = String.format(msg, link);
            WxMpKefuMessage wxMpKefuMessage = new TextBuilder().toUser(user.getOpenid()).content(msg).build();
            pushService.addWxKefuMessagePushTask(wxMpService, wxMpKefuMessage, to - (userRecentlyRead.getUpdatetime()));
        }
    }
}
