package com.book.server.controller;

import com.book.dao.VO.SunVO;
import com.book.dao.polardb.entity.SunChannel;
import com.book.dao.polardb.entity.SunOrder;
import com.book.dao.polardb.entity.SunOrderCollect;
import com.book.dao.polardb.entity.SunUserCollect;
import com.book.server.common.entity.PageResult;
import com.book.server.common.entity.Result;
import com.book.server.config.SunTypeEnum;
import com.book.server.service.SunDataService;
import com.book.server.vo.SunNotifyRequest;
import com.book.server.vo.SunRequest;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.codec.digest.DigestUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.websocket.server.PathParam;
import java.util.UUID;

/**
 * SUN
 * created in 2021/9/27
 * Project: book-store
 *
 * @author win7
 */
@RestController
@RequestMapping("/api/sun")
@Slf4j
public class SunDataController extends BaseController {
    @Autowired
    private SunDataService sunDataService;


    /**
     * getData
     * @param type
     * @param request
     * @return
     */
    @PostMapping("/getData/{type}")
    public Result getData(@PathVariable("type") String type, @RequestBody SunRequest request) {
        SunTypeEnum sunTypeEnum = SunTypeEnum.valueOf(type);
        return sunDataService.requestData(request, sunTypeEnum);

    }

    /**
     * pullData
     * @param request
     * @return
     */
    @PostMapping("/pullData")
    public String pushData(@RequestBody SunNotifyRequest request) {
        sunDataService.pullData(request);
        String s = "{\"code\":200,\"msg\":\"ok\"}";
        return s;

    }

    /**
     * 获取 渠道
     *
     * @param sunVO
     * @return
     */
    @PostMapping("/pageChannel")
    public Result<PageResult<SunChannel>> pageSunChannel(@RequestBody SunVO sunVO) {
        return Result.success(sunDataService.pageSunChannel(sunVO));
    }

    /**
     * 获取 订单明细表
     *
     * @param sunVO
     * @return
     */
    @PostMapping("/pageOrder")
    public Result<PageResult<SunOrder>> pageOrder(@RequestBody SunVO sunVO) {
        return Result.success(sunDataService.pageSunOrder(sunVO));
    }

    /**
     * 获取 订单统计表
     *
     * @param sunVO
     * @return
     */
    @PostMapping("/pageOrderCollect")
    public Result<PageResult<SunOrderCollect>> pageOrderCollect(@RequestBody SunVO sunVO) {
        return Result.success(sunDataService.pageOrderCollect(sunVO));
    }

    /**
     * 获取 用户统计表
     *
     * @param sunVO
     * @return
     */
    @PostMapping("/pageUserCollect")
    public Result<PageResult<SunUserCollect>> pageUserCollect(@RequestBody SunVO sunVO) {
        return Result.success(sunDataService.pageUserCollect(sunVO));
    }


    public static void main(String[] args) {
        System.out.println(UUID.randomUUID().toString());
    }

}
