package com.book.server.controller;

import com.book.dao.cps.entity.UserReadConfig;
import com.book.dao.cps.pojo.User;
import com.book.server.common.entity.Result;
import com.book.server.service.CacheService;
import com.book.server.service.UserService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

/**
 * 用户
 */
@RestController
@RequestMapping("/api/user")
@Slf4j
public class UserController extends BaseController {
    @Autowired
    private UserService userService;

    @Autowired
    private CacheService cacheService;

    /**
     * 获取用户基本信息
     *
     * @param openId 微信 openId
     * @return
     */
    @GetMapping("/getUser")
    public Result<User> getUserByOpenId(String openId) {
        User user = userService.getUserByOpenId(openId);
        return Result.byObject(user);
    }

    /**
     * 根据 id 获取用户
     *
     * @param userId
     * @return
     */
    @GetMapping("/getUserByUserId")
    public Result<User> getUserByUserId(Long userId) {
        User user = cacheService.getUserByIdCache(userId);
        return Result.byObject(user);
    }

    /**
     * 获取用户阅读设置：颜色、字体
     *
     * @return
     */
    @GetMapping("/getReadConfig")
    public Result<UserReadConfig> getUserReadConfigByUserId(Long userId) {
        return Result.success(cacheService.getUserReadConfigCache(userId));
    }

    /**
     * 设置用户阅读习惯：颜色、字体
     *
     * @return
     */
    @PostMapping("/setReadConfig")
    public Result setUserReadConfigByUserId(@RequestBody UserReadConfig readConfig) {
        if (readConfig.getUserId() == null) {
            return Result.failure();
        }
        return Result.success(userService.setUserReadConfigByUserId(readConfig));
    }

    /**
     * 获取域名
     * @return
     */
    @GetMapping("/getUrl")
    public Result getUrl() {
        String serverName = request.getServerName();
        return Result.byObject(serverName);
    }
}
