package com.book.server.myevent;

import com.book.dao.VO.BookContentVO;
import com.book.dao.cpsshard.entity.UserRecentlyRead;
import com.book.dao.cpsshard.mapper.UserRecentlyReadMapper;
import com.book.dao.cpsshard.pojo.example.UserRecentlyReadExample;
import com.book.dao.utils.DateUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.event.EventListener;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.concurrent.LinkedBlockingQueue;

@Component
@Slf4j
public class AsyncListener {

    @Autowired
    private UserRecentlyReadMapper userRecentlyReadMapper;

    @EventListener
    @Async  //异步方法
    public void updateUserRead(UserRecentReadEvent event) {
        log.info("阅读记录: " + event.toString());
        try {
            BookContentVO bookContentVO = event.bookContentVO;
            //尝试更新阅读记录
            UserRecentlyRead userRecentlyRead = UserRecentlyRead.builder()
                    .chapterId(bookContentVO.getContentId())
                    .chapterName(bookContentVO.getContentName())
                    .updatetime(DateUtils.getNow())
                    .build();
            userRecentlyRead.setChapterId(bookContentVO.getContentId());
            userRecentlyRead.setChapterName(bookContentVO.getContentName());
            userRecentlyRead.setUpdatetime(DateUtils.getNow());
            int i = userRecentlyReadMapper.updateByExampleSelective(userRecentlyRead,
                    UserRecentlyReadExample.newAndCreateCriteria()
                            .andBookIdEqualTo(bookContentVO.getBookId())
                            .andUserIdEqualTo(bookContentVO.getUserId())
                            .example()
                    ,
                    UserRecentlyRead.Column.chapterId,
                    UserRecentlyRead.Column.chapterName,
                    UserRecentlyRead.Column.updatetime
            );

            //如果不存在，则新增
            if (i <= 0) {
                userRecentlyReadMapper.insert(UserRecentlyRead.builder()
                        .userId(bookContentVO.getUserId())
                        .bookId(bookContentVO.getBookId())
                        .chapterId(bookContentVO.getContentId())
                        .chapterName(bookContentVO.getContentName())
                        .createtime(DateUtils.getNow())
                        .updatetime(DateUtils.getNow())
                        .bookShelfAdd((byte) 0)
                        .bookShelfFlag((byte) 0)
                        .build());
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    @EventListener
    @Async  //异步方法
    public void testListener(TestEvent event) {
        try {
            System.out.println(event.string);

        } catch (Exception e) {
            e.printStackTrace();
        }
    }
}
