package com.book.server.service;

import com.book.dao.VO.BlockRes;
import com.book.server.common.util.CacheUtil;
import com.book.server.utils.RedisUtil;
import com.google.common.cache.CacheBuilder;
import com.google.common.cache.CacheLoader;
import com.google.common.cache.LoadingCache;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.List;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeUnit;

@Component
public class BlockCacheGuava {
    private LoadingCache<Integer, List<BlockRes>> blockResCache;

    @Autowired
    private CacheService cacheService;

    @PostConstruct
    public void init() {
        blockResCache = CacheBuilder.newBuilder()
                .maximumSize(10) //设置缓存最大容量
                .refreshAfterWrite(5, TimeUnit.MINUTES)
                .build(new CacheLoader<Integer, List<BlockRes>>() {
                    @Override
                    public List<BlockRes> load(Integer key) throws Exception {
                        return cacheService.getBlockByPageId(key);
                    }
                });
    }

    public List<BlockRes> getBlockRes(Integer key) throws ExecutionException {
        return blockResCache.get(key);
    }

    @Autowired
    private RedisUtil redisUtil;

    public void clearCache() {
        blockResCache.refresh(1);
        blockResCache.refresh(2);
        redisUtil.removePattern(CacheUtil.PRE + "*");
    }
}
