package com.book.server.service;

import com.book.dao.VO.*;
import com.book.dao.cps.entity.EditBook;
import com.book.dao.cps.entity.UploadBook;
import com.book.dao.cps.entity.UserReadConfig;
import com.book.dao.cps.mapper.*;
import com.book.dao.cps.pojo.*;
import com.book.dao.cps.pojo.example.BookCategoryExample;
import com.book.dao.cps.pojo.example.ManageBlockExample;
import com.book.dao.cps.pojo.example.SearchKeywordExample;
import com.book.dao.cps.pojo.example.UploadBookExample;
import com.book.server.annotation.CacheFind;
import com.book.server.config.Const;
import com.book.server.service.impl.BookServiceImpl;
import com.book.server.utils.HttpTool;
import com.google.gson.JsonArray;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.TreeSet;
import java.util.stream.Collectors;

@Service
public class CacheService {
    @Autowired
    private BookMapper bookMapper;
    @Autowired
    private BookCategoryMapper bookCategoryMapper;
    @Autowired
    private BookMapper2 bookMapper2;

    @Autowired
    private ManageBlockMapper manageBlockMapper;

    @Autowired
    private SearchKeywordMapper searchKeywordMapper;
    @Autowired
    private ManageBlockResource2Mapper manageBlockResource2Mapper;

    @Autowired
    private BookServiceImpl bookService;

    @Autowired
    private ConfigMapper configMapper;

    @CacheFind(key = "book")
    public Book getBookById(Long id) {
        return bookMapper.selectByPrimaryKey(id);
    }

    @CacheFind(key = "category")
    public String getCategoryName(Integer id) {
        List<BookCategory> bookCategories = bookCategoryMapper.selectByExample(BookCategoryExample.newAndCreateCriteria().example());
        String name = "";
        for (BookCategory x : bookCategories) {
            if (x.getId() == id) {
                name = x.getName();
            }
        }
        return name;
    }

    @CacheFind(key = "recommend")
    public List<BookRes> getRecommendFromCache(String sex, int page, int size) {
        QueryVO queryVO = new QueryVO();
        queryVO.setSex(sex);
        queryVO.setPage(page);
        queryVO.setSize(size);
        return bookMapper2.novelEndRecommend(queryVO,
                queryVO.getPage() * queryVO.getSize(), queryVO.getSize());
    }

    @CacheFind(key = "recommend", delete = true)
    public void deleteRecommendFromCache(String sex, int page, int size) {
    }

    public ArrayList<BlockRes> getBlockByPageId(Integer pageId) {
        //没有缓存
        ArrayList<BlockRes> list = new ArrayList<>();
        // 获取板块
        ManageBlockExample manageBlockExample = ManageBlockExample.newAndCreateCriteria()
                .andPageIdEqualTo(pageId)
                .example();
        manageBlockExample.orderBy(ManageBlock.Column.weigh.desc());
        List<ManageBlock> manageBlocks = manageBlockMapper.selectByExample(manageBlockExample);

        // 获取已有版本的所有书籍
        List<Integer> blockIds = manageBlocks.stream().map(ManageBlock::getId).collect(Collectors.toList());
        List<ManageBlockResourceRes> blockResourceResList = manageBlockResource2Mapper.getBooks(blockIds);

        // 合并相同 block 书籍
        HashMap<Integer, TreeSet<ManageBlockResourceRes>> map = new HashMap<>();
        blockResourceResList.forEach(x -> {
            TreeSet<ManageBlockResourceRes> set = map.computeIfAbsent(x.getBlockId(), k -> new TreeSet<>((o1, o2) -> o2.getWeigh() - o1.getWeigh()));
            set.add(x);
        });

        manageBlocks.forEach(x -> {
            BlockRes blockRes = new BlockRes();
            blockRes.setName(x.getName());
            blockRes.setType(x.getType());
            blockRes.setSecondName(x.getSecondName());
            blockRes.setResources(map.getOrDefault(x.getId(), new TreeSet<>()));
            list.add(blockRes);
        });
        return list;
    }

    @Autowired
    private UploadBookMapper uploadBookMapper;

    @CacheFind(key = "chapters")
    public List<Chapter> getChaptersByBookIdCache(Long bookId) {
        List<Chapter> list = new ArrayList<>();
        //免费章节
        int freeNum = 0;
        Book bookById = getBookById(bookId);
        if (bookById.getFreeChapterNum() == null) {
            freeNum = bookService.getDefaultFreeChapterNum();
        } else {
            freeNum = bookById.getFreeChapterNum();
        }
        //如果是本地书籍
        if (isUploadBook(bookId)) {
            List<UploadBook> uploadBooks = uploadBookMapper.selectByExample(
                    UploadBookExample.newAndCreateCriteria()
                            .andBookIdEqualTo(bookId)
                            .example()
            );
            int finalFreeNum = freeNum;
            uploadBooks.forEach(x -> {
                Chapter chapter = new Chapter();
                chapter.setBookId(x.getBookId().toString());
                chapter.setContentId(x.getChapterId().toString());
                chapter.setName(x.getChapterName());
                if (x.getNum() >= finalFreeNum) {
                    chapter.setReadAble("1");
                } else {
                    chapter.setReadAble("0");
                }
                chapter.setWords(x.getWords().toString());
                chapter.setNum(x.getNum().toString());
                list.add(chapter);
            });
            return list;
        }

        final String chaptersUrl = "http://new.emeixs.com/index.php/Home/Interface/chapters/bid/" + bookId;
        String result = HttpTool.sendGet(chaptersUrl, "");
        if (StringUtils.isEmpty(result)) {
            return list;
        }
        JsonParser parser = new JsonParser();
        JsonArray jsonArray = parser.parse(result).getAsJsonArray();
        Book book = getBookById(bookId);
        if (book == null) {
            return list;
        }
        Integer freeChapterNum = book.getFreeChapterNum();

        if (freeChapterNum == null) {
            freeChapterNum = 20;
        }
        for (int i = 0; i < jsonArray.size(); i++) {
            JsonObject jsonObject = jsonArray.get(i).getAsJsonObject();
            Chapter chapter = new Chapter();
            chapter.setReadAble("1");
            chapter.setBookId(jsonObject.get("book_id").getAsString());
            chapter.setContentId(jsonObject.get("content_id").getAsString());
            chapter.setIsPay(jsonObject.get("ispay").getAsString());
            chapter.setName(jsonObject.get("name").getAsString());
            chapter.setNum(jsonObject.get("num").getAsString());
            chapter.setType(jsonObject.get("type").getAsString());
            chapter.setVolumeId(jsonObject.get("volume_id").getAsString());
            chapter.setWords(jsonObject.get("words").getAsString());
            if (i + 1 <= freeChapterNum) {
                chapter.setReadAble("0");
            }
            list.add(chapter);
        }
        return list;
    }

    public Boolean isUploadBook(Long bookId) {
        return bookId >= Const.bookIdMin && bookId <= Const.bookInMax;
    }

    @CacheFind(key = "defaultFreeChapterNum")
    public int setBook_free_chapter_num() {
        int book_free_chapter_num = Integer.parseInt(configMapper.selectByName("book_free_chapter_num").getValue());
        return book_free_chapter_num;
    }

    @Autowired
    private UserMapper userMapper;

    public User getUserByIdCache(Long id) {
        return bookService.getUserById(id);
    }

    @CacheFind(key = "freeCoin")
    public Integer getFreeCoinCache(Long id) {
        return bookService.getFreeCoin(id);
    }

    @CacheFind(key = "freeCoin", update = true)
    public Integer updateFreeCoinCache(Long id) {
        return bookService.getFreeCoin(id);
    }


    @CacheFind(key = "user", update = true)
    public User updateUserByIdCache(Long id) {
        return bookService.getUserById(id);
    }

    @CacheFind(key = "searchKeyword")
    public List<SearchKeyword> getSearchKeywordsCache(String sex) {
        SearchKeywordExample example = SearchKeywordExample.newAndCreateCriteria()
                .andSexEqualTo(sex)
                .example();
        example.orderBy(SearchKeyword.Column.weigh.desc());
        List<SearchKeyword> searchKeywords = searchKeywordMapper.selectByExample(example);
        return searchKeywords;
    }

    @CacheFind(key = "searchKeyword", delete = true)
    public void deleteSearchKeywordsCache(String sex) {
    }

    @CacheFind(key = "smartRecommand")
    public List<BookRes> setSmartRecommandCache(String sex) {
        QueryVO queryVO = new QueryVO();
        queryVO.setSex(sex);
        List<BookRes> bookRes = bookMapper2.smartRecommand(queryVO);
        return bookRes;
    }

    @CacheFind(key = "smartRecommand", delete = true)
    public void deleteSmartRecommandCache(String sex) {
    }

    @CacheFind(key = "topRecommend")
    public List<BookRes> setTopRecommendCache(String sex) {
        QueryVO queryVO = new QueryVO();
        queryVO.setSex(sex);
        List<BookRes> bookRes = bookMapper2.topRecommend(queryVO, 0, 20);
        return bookRes;
    }

    @CacheFind(key = "topRecommend", delete = true)
    public void deleteTopRecommendCache(String sex) {
    }

    @Autowired
    private UserService userService;

    @CacheFind(key = "userReadConfig")
    public UserReadConfig getUserReadConfigCache(Long userId) {
        return userService.getUserReadConfigByUserId(userId);
    }

    @CacheFind(key = "userReadConfig", delete = true)
    public void delUserReadConfigCache(Long userId) {
    }

    @CacheFind(key = "edit_book")
    public EditBook getEditBookCache(Long bookId, Long chapterId) {
        return bookService.getEditBookContent(bookId, chapterId);
    }

    @CacheFind(key = "edit_book", update = true)
    public EditBook updateEditBookCache(Long bookId, Long chapterId) {
        return bookService.getEditBookContent(bookId, chapterId);
    }

    @CacheFind(key = "bookContent")
    public BookContent getBookContentCache(Long bookId, Long chapterId) {
        return bookService.getBookContent(bookId, chapterId);
    }
    @CacheFind(key = "guide_chapter")
    public Integer getBook_guide_chapter_idx(String appId, Integer bookId) {
        return bookService.getBook_guide_chapter_idx(appId, bookId);
    }

    @CacheFind(key = "qrcodeUrl")
    public String getQrcodeUrl(String appId) {
        return bookService.getQrcodeUrl(appId);
    }
}
