package com.book.server.service.impl;


import com.book.dao.cps.entity.EditBook;
import com.book.dao.cps.entity.UploadBook;
import com.book.dao.cps.mapper.*;
import com.book.dao.cps.pojo.*;
import com.book.dao.cpsshard.entity.Consume;
import com.book.dao.cps.pojo.Recharge;
import com.book.dao.cpsshard.entity.UserRecentlyRead;
import com.book.dao.cpsshard.mapper.ConsumeMapper;
import com.book.dao.cps.mapper.RechargeMapper;
import com.book.dao.cpsshard.mapper.UserRecentlyReadMapper;
import com.book.dao.cpsshard.pojo.example.ConsumeExample;
import com.book.dao.cps.pojo.example.RechargeExample;
import com.book.dao.cpsshard.pojo.example.UserRecentlyReadExample;
import com.book.server.annotation.CacheFind;
import com.book.server.common.entity.Result;
import com.book.server.common.entity.ResultCode;
import com.book.dao.utils.DateUtils;
import com.book.server.common.util.CacheUtil;
import com.book.server.common.util.JsonUtils;
import com.book.dao.cps.pojo.example.*;
import com.book.dao.VO.*;
import com.book.server.config.Const;
import com.book.server.myevent.UserRecentReadEvent;
import com.book.server.service.BookService;
import com.book.server.service.BlockCacheGuava;
import com.book.server.service.CacheService;
import com.book.server.utils.HttpTool;
import com.book.server.utils.RedisUtil;
import com.google.gson.JsonArray;
import com.google.gson.JsonElement;
import com.google.gson.JsonObject;
import com.google.gson.JsonParser;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.PostConstruct;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.nio.charset.StandardCharsets;
import java.util.*;
import java.util.concurrent.ExecutionException;
import java.util.stream.Collectors;

@Service
@Slf4j
public class BookServiceImpl implements BookService {
    @Autowired
    private BookMapper bookMapper;
    @Autowired
    private BookMapper2 bookMapper2;
    @Autowired
    private CacheService cacheService;

    @Autowired
    private ManageBlockMapper manageBlockMapper;

    @Autowired
    private BookCategoryMapper bookCategoryMapper;
    @Autowired
    private SearchKeywordMapper searchKeywordMapper;
    @Autowired
    private ManageBlockResource2Mapper manageBlockResource2Mapper;


    public List<BookCategory> getAllCategory() {
        List<BookCategory> bookCategories = bookCategoryMapper.selectByExample(
                BookCategoryExample.newAndCreateCriteria().example()
        );
        return bookCategories;
    }


    @Override
    public List<BookRes> query(QueryVO queryVO) {
        List<BookRes> bookResList = new ArrayList<>();
        if (queryVO.getBookId() != null) {
            Book bookById = cacheService.getBookById(queryVO.getBookId());
            String categoryName = cacheService.getCategoryName(bookById.getBookCategoryId());
            BookRes bookRes = new BookRes();
            BeanUtils.copyProperties(bookById, bookRes);
            bookRes.setCategoryName(categoryName);
            bookResList.add(bookRes);
        } else {
            queryVO.setQuery("%" + queryVO.getQuery() + "%");
            bookResList = bookMapper2.queryBookRes(queryVO, queryVO.getPage() * queryVO.getSize(), queryVO.getSize());
        }
        return bookResList;
    }

    @Override
    public List<BookRes> recommendByRead(QueryVO queryVO) {
        ArrayList<BookRes> bookRes = new ArrayList<>();
        if (queryVO.getPage() == 5) {
            return bookRes;
        }
        return cacheService.getRecommendFromCache(queryVO.getSex(), queryVO.getPage(), queryVO.getSize());
    }

    @Autowired
    private BlockCacheGuava blockCacheGuava;

    @Override
    public List<BlockRes> getBlockByPageId(Integer pageId) throws ExecutionException {
        return blockCacheGuava.getBlockRes(pageId);
    }

    @Override
    public void insertBooks() {

        //插入书籍基本信息
        String bookInfoUrl = "http://new.emeixs.com/index.php/Home/Interface/listing";

        String jsonStr = HttpTool.sendGet(bookInfoUrl, "");
        JsonObject jsonObject = JsonUtils.getObject(jsonStr, JsonObject.class);
        JsonArray list = jsonObject.getAsJsonArray("list");

        Map<String, String> typeMap = new HashMap<String, String>();
        typeMap.put("15", "豪门总裁");
        typeMap.put("16", "婚恋家庭");
        typeMap.put("17", "豪门虐恋");
        typeMap.put("18", "甜宠萌宝");
        typeMap.put("19", "重生逆袭");
        typeMap.put("20", "穿越重生");
        typeMap.put("21", "权谋宫斗");
        typeMap.put("22", "经商种田");
        typeMap.put("23", "出版");
        typeMap.put("24", "其他");
        List<Book> bookList = new ArrayList<>();
        for (int i = 0; i < list.size(); i++) {
            JsonElement jsonElement = list.get(i);
            JsonObject bookObject = jsonElement.getAsJsonObject();
            String author_name = bookObject.get("author_name").getAsString();
            String book_brief = bookObject.get("book_brief").getAsString();
            String book_id = bookObject.get("book_id").getAsString();
            String book_name = bookObject.get("book_name").getAsString();
            String chapter = bookObject.get("chapter").getAsString();
            String charge = bookObject.get("charge").getAsString();
            String cover = bookObject.get("cover").getAsString();
            String ctype_id = bookObject.get("ctype_id").getAsString();
            String gender = bookObject.get("gender").getAsString();
            String is_show = bookObject.get("is_show").getAsString();
            String state = bookObject.get("state").getAsString();
            String type_id = bookObject.get("type_id").getAsString();
            String words = bookObject.get("words").getAsString();

            //章节列表
            String chaptersStr = HttpTool.sendGet("http://new.emeixs.com/index.php/Home/Interface/chapters/bid/" + book_id, "");
            JsonParser parser = new JsonParser();
            JsonArray jsonArray = parser.parse(chaptersStr).getAsJsonArray();
            JsonObject firstChapters = jsonArray.get(1).getAsJsonObject();
            String first_chapter_id = firstChapters.get("content_id").getAsString();
            String first_chapter_name = firstChapters.get("name").getAsString();
            JsonObject lastChapters = jsonArray.get(jsonArray.size() - 1).getAsJsonObject();
            String last_chapter_id = lastChapters.get("content_id").getAsString();
            String last_chapter_name = lastChapters.get("name").getAsString();


            Book book = new Book();
            book.setId(Long.parseLong(book_id));
            book.setBookCategoryId(Integer.parseInt(type_id));
            book.setName(book_name);
            book.setRealname(book_name);
            book.setAuthor(author_name);
            book.setImage(cover);
            book.setDescription(book_brief);
            book.setSex(gender);
            book.setBillingType(String.valueOf(Integer.parseInt(charge) + 1));
            book.setWordCount(words);
            book.setChapterNum(Integer.valueOf(chapter));
//            book.setLastChapterUtime(Integer.parseInt(System.currentTimeMillis()+""));
            book.setCreatetime(Integer.parseInt(System.currentTimeMillis() / 1000 + ""));
            book.setCpId(Integer.parseInt(book_id));
            book.setCpName(book_name);
            book.setTags(typeMap.get(ctype_id));
            book.setState("1");
            book.setPrice(45);
            book.setArticleChapterOrder(4);
            book.setAttentChapterOrder(4);
            book.setCornerMark("hot");
            book.setIsFinish(String.valueOf(Integer.parseInt(state) - 1));
            book.setReferralNum(new Byte((byte) 5));
            book.setPutAdSet(new Byte((byte) 1));
            book.setClassifyWhiteList(new Byte((byte) 0));
            book.setCheckRank(new Byte((byte) 0));
            book.setRank(new Byte((byte) 6));
            book.setExpireTime(Integer.MAX_VALUE);
            book.setLastChapterUtime(Integer.parseInt(System.currentTimeMillis() / 1000 + ""));
            book.setAppPrice(45);
            book.setFirstChapterId(Long.parseLong(first_chapter_id));
            book.setFirstChapterName(first_chapter_name);
            book.setLastChapterId(Long.parseLong(last_chapter_id));
            book.setLastChapterName(last_chapter_name);
            book.setCansee("1");
            book.setRank(new Byte((byte) 6));

            bookList.add(book);


        }

        for (Book book : bookList) {
            int i = new Random().nextInt(bookList.size());
            book.setRecommandBookId(bookList.get(i).getId());
            System.out.println(JsonUtils.toJsonStr(book));
            bookMapper.insert(book);


        }

    }

    @Override
    public List<Chapter> getChaptersByBookId(Long bookId) {
        if (bookId == null) {
            throw new IllegalArgumentException("bookId cannot be null or empty");
        }

        //缓存
        return cacheService.getChaptersByBookIdCache(bookId);
    }


    @Override
    public List<SearchKeyword> getSearchKeywordBySex(String sex) {
        return cacheService.getSearchKeywordsCache(sex);
    }


    @Override
    public List<BookRes> topRecommend(QueryVO queryVO) {
        List<BookRes> bookRes = cacheService.setTopRecommendCache(queryVO.getSex());
        return getRandom(bookRes);
    }


    @Override
    public List<BookRes> smartRecommand(QueryVO queryVO) {
        return cacheService.setSmartRecommandCache(queryVO.getSex());
    }


    @Autowired
    private ConfigMapper configMapper;
    @Autowired
    private AdminConfigMapper adminConfigMapper;

    @Autowired
    private ConsumeMapper consumeMapper;
    @Autowired
    private UserMapper userMapper;

    @Autowired
    private RechargeMapper rechargeMapper;

    @Transactional
    @Override
    public Result<BookContent> getContentByContentId(BookContentVO bookContentVO) {
        BookContent content = null;
        Book book = cacheService.getBookById(bookContentVO.getBookId());
        //是否是付费章节
        Integer freeChapterNum = book.getFreeChapterNum();
        if (freeChapterNum == null) { //查看默认配置
            freeChapterNum = getDefaultFreeChapterNum();
        }
        if (bookContentVO.getNum() >= freeChapterNum) {
            //收费
            //是否已购买
            Consume consume = consumeMapper.selectOneByExample(ConsumeExample.newAndCreateCriteria()
                    .andUserIdEqualTo(bookContentVO.getUserId())
                    .andBookIdEqualTo(bookContentVO.getBookId())
                    .andChapterIdEqualTo(bookContentVO.getContentId())
                    .example()
            );
            if (consume == null) { // 没有购买记录
                User user = cacheService.getUserByIdCache(bookContentVO.getUserId());
                //查询免费看点
                List<Recharge> freeRecharges = getFreeCharges(bookContentVO.getUserId());
                int freeCount = getFreeCountFromCharges(freeRecharges);
                //免费是否足够
                if (freeCount >= book.getPrice()) {
                    //扣除一部分免费的
                    int left = book.getPrice();
                    for (Recharge recharge : freeRecharges) {
                        if (left <= recharge.getRemainFreeKandian()) {
                            subtractFreeRecharge(left, recharge);
                            break;
                        } else {
                            subtractFreeRecharge(recharge.getRemainFreeKandian(), recharge);
                            left -= recharge.getRemainFreeKandian();
                        }
                    }
                    // 记录到消费表
                    addConsumer(bookContentVO, book, left, 0);
                    // 更新免费看点到 user
                    int i = userMapper.updateUserKandian(user, user.getKandian(), freeCount - book.getPrice(), DateUtils.getNow());
                    checkUpdate(i);
                    cacheService.updateFreeCoinCache(user.getId());

                } else if (user.getKandian() + freeCount >= book.getPrice()) { //免费+充值足够
                    Integer left = user.getKandian() + freeCount;
                    if (freeCount != 0) {
                        //扣除所有免费
                        for (Recharge recharge : freeRecharges) {
                            subtractFreeRecharge(recharge.getRemainFreeKandian(), recharge);
                        }
                    }
                    //扣除充值的
                    int kandian = user.getKandian() - (book.getPrice() - freeCount);

                    // 记录到消费表
                    addConsumer(bookContentVO, book, freeCount, book.getPrice() - freeCount);

                    // 更新到 user
                    int i = userMapper.updateUserKandian(user, kandian, 0, DateUtils.getNow());
                    checkUpdate(i);
                    cacheService.updateFreeCoinCache(user.getId());

                } else {
                    return Result.failure(ResultCode.NOMONEY);
                }
            }

        }
        //获取章节内容
        content = cacheService.getBookContentCache(bookContentVO.getBookId(), bookContentVO.getContentId());
        // 添加到阅读记录
        if (content != null) {
            applicationEventPublisher.publishEvent(new UserRecentReadEvent(this, bookContentVO));
            //applicationEventPublisher.publishEvent(new TestEvent(this, "!!!!!!!"));
        }
        return Result.byObject(content);
    }

    public int getFreeCountFromCharges(List<Recharge> freeRecharges) {
        Integer freeCount = 0;
        if (freeRecharges != null) {
            freeCount = freeRecharges.stream().mapToInt(x -> x.getFreeKandian()).sum();
        }
        return freeCount;
    }

    public List<Recharge> getFreeCharges(Long userId) {
        RechargeExample example = RechargeExample.newAndCreateCriteria()
                .andUserIdEqualTo(userId)
                .andFreeEndtimeGreaterThan(DateUtils.getNow())
                .andRemainFreeKandianGreaterThan(0)
                .example();
        example.orderBy(Recharge.Column.freeEndtime.desc());
        List<Recharge> freeRecharges = rechargeMapper.selectByExample(example);
        return freeRecharges;
    }

    @Autowired
    private ApplicationEventPublisher applicationEventPublisher;

    public BookContent getBookContent(Long bookId, Long contentId) {
        BookContent bookContent;
        // 修改过的章节
        EditBook editBook = cacheService.getEditBookCache(bookId, contentId);

        if (editBook != null) {
            return getBookContentFromVO(bookId, contentId, editBook.getChapterName(), editBook.getContent());
        }
        if (cacheService.isUploadBook(bookId)) {
            // 自定义的书籍
            bookContent = getBookContentFromDB(bookId, contentId);
        } else {
            bookContent = getBookContentFromRemote(bookId, contentId);
        }
        return bookContent;
    }


    private void checkUpdate(int i) {
        if (i <= 0) {
            throw new RuntimeException("请稍后重试");
        }
    }

    /**
     * 消费记录
     *
     * @param bookContentVO
     * @param book
     * @param freeKandian
     * @param kandian
     */
    @Transactional
    void addConsumer(BookContentVO bookContentVO, Book book, int freeKandian, int kandian) {
        int i = consumeMapper.insert(Consume.builder()
                .userId(bookContentVO.getUserId())
                .type("1") // 订购章节
                .bookId(book.getId())
                .bookName(book.getName())
                .chapterId(bookContentVO.getContentId())
                .chapterName(bookContentVO.getContentName())
                .ddKandian(kandian)
                .ddFreeKandian(freeKandian)
                .createtime(DateUtils.getNow())
                .updatetime(DateUtils.getNow())
                .businessLine((byte) 0)
                .build());
        checkUpdate(i);
    }

    /**
     * 减去多少免费看点
     *
     * @param reduceNum 消费多少免费看点
     * @param recharge
     */
    @Transactional()
    void subtractFreeRecharge(int reduceNum, Recharge recharge) {
        int i = rechargeMapper.updateByExampleSelective(
                Recharge.builder().remainFreeKandian(recharge.getRemainFreeKandian() - reduceNum).build(),
                RechargeExample.newAndCreateCriteria()
                        .andIdEqualTo(recharge.getId())
                        .andRemainFreeKandianEqualTo(recharge.getRemainFreeKandian()).example(),
                Recharge.Column.remainFreeKandian
        );
        checkUpdate(i);
    }

    /**
     * 获取版权方章节内容
     *
     * @return
     */
    public BookContent getBookContentFromRemote(Long bookId, Long contentId) {
        String contentUrl = "http://new.emeixs.com/index.php/Home/Interface/content/bid/%s/cid/%s";
        if (bookId == null || contentId == null) {
            return null;
        }
        BookContent content = null;
        try {
            content = new BookContent();
            String url = String.format(contentUrl, bookId, contentId);
            String result = HttpTool.sendGet(url, "");
            JsonObject jsonObject = JsonUtils.getObject(result, JsonObject.class);

            content.setContent(jsonObject.get("content").getAsString());
            content.setBookId(jsonObject.get("book_id").getAsString());
            content.setName(jsonObject.get("name").getAsString());
            content.setChapterId(jsonObject.get("CONTENT_ID").getAsString());
            content.setIsPay(jsonObject.get("ispay").getAsString());
        } catch (Exception e) {
            log.error(e.getMessage());
            return null;
        }
        return content;
    }

    @Autowired
    private UserRecentlyReadMapper userRecentlyReadMapper;

    @Override
    public UserRecentlyRead getRecentReadByUserIdAndBookId(UserRecentlyRead userRecentlyRead) {
        UserRecentlyReadExample example = UserRecentlyReadExample.newAndCreateCriteria()
                .andUserIdEqualTo(userRecentlyRead.getUserId())
                .andBookIdEqualTo(userRecentlyRead.getBookId())
                .example();
        return userRecentlyReadMapper.selectOneByExample(example);
    }

    @Override
    public List<UserRecentlyReadRes> getRecentReadByUserId(UserRecentReadVO vo) {
        ArrayList<UserRecentlyReadRes> res = new ArrayList<>();
        UserRecentlyReadExample example = UserRecentlyReadExample.newAndCreateCriteria()
                .andUserIdEqualTo(vo.getUserId())
                .example();
        example.orderBy(UserRecentlyRead.Column.updatetime.desc());
        example.page(vo.getPage(), vo.getSize());
        List<UserRecentlyRead> userRecentlyReads = userRecentlyReadMapper.selectByExample(example);

        if (userRecentlyReads == null || userRecentlyReads.size() == 0) {
            return res;
        }

        //获取 bookId 和 cover
        List<Long> bookIds = userRecentlyReads.stream().map(x -> x.getBookId()).collect(Collectors.toList());
        List<Book> books = bookMapper.selectByExampleSelective(
                BookExample.newAndCreateCriteria()
                        .andIdIn(bookIds).example()
                , Book.Column.id, Book.Column.name, Book.Column.image
        );
        HashMap<Long, Book> map = new HashMap<>();
        for (Book book : books) {
            map.put(book.getId(), book);
        }

        // 填充 res
        for (UserRecentlyRead userRecentlyRead : userRecentlyReads) {
            UserRecentlyReadRes recentlyReadRes = new UserRecentlyReadRes();
            BeanUtils.copyProperties(userRecentlyRead, recentlyReadRes);
            Book book = map.get(userRecentlyRead.getBookId());
            if (book != null) {
                recentlyReadRes.setBookName(book.getName());
                recentlyReadRes.setImage(book.getImage());
            }
            res.add(recentlyReadRes);
        }

        return res;
    }

    @Override
    public List<Long> getUserConsumerChapter(BookContentVO bookContentVO) {
        List<Consume> consumes = consumeMapper.selectByExampleSelective(ConsumeExample.newAndCreateCriteria()
                        .andUserIdEqualTo(bookContentVO.getUserId())
                        .andBookIdEqualTo(bookContentVO.getBookId())
                        .example(),
                Consume.Column.chapterId
        );
        List<Long> collect = new ArrayList<>();
        if (consumes != null) {
            collect = consumes.stream().map(Consume::getChapterId).collect(Collectors.toList());
        }
        return collect;
    }

    @Autowired
    private UploadBookMapper uploadBookMapper;

    @Transactional
    @Override
    public void addFile(MultipartFile file, Long bookId) throws IOException {

        // 测试先清理数据
        uploadBookMapper.deleteByExample(UploadBookExample.newAndCreateCriteria().andBookIdEqualTo(bookId).example());
        // 读取所有行
        BufferedReader bf = new BufferedReader(new InputStreamReader(file.getInputStream(), StandardCharsets.UTF_8));
        String strTmp;
        LinkedList<String> list = new LinkedList<>();
        while ((strTmp = bf.readLine()) != null) {
            if (StringUtils.isNotBlank(strTmp) && !strTmp.startsWith("更新时间：")) {
                list.add(strTmp);
            }
        }
        StringBuilder content = new StringBuilder();
        int chapterNum = 0;
        // 获取内容简介，并保存到数据库
        if (!list.isEmpty() && list.pop().startsWith("内容简介")) {
            while (!list.isEmpty()) {
                String pop = list.peek();
                if (pop != null && pop.startsWith(" ")) {
                    content.append(pop.trim()).append("\r\n");
                    list.pop();
                } else {
                    break;
                }
            }
        }

        int wordCount = 0;
        Long firstChapterId = 0L;
        Long lastChapterId = 0L;
        String firstChapterName = "";
        String lastChapterName = "";

        // 获取默认免费章节数
        int defaultFreeChapterNum = getDefaultFreeChapterNum();
        int i = 1;
        ArrayList<UploadBook> books = new ArrayList<>();
        while (!list.isEmpty()) {
            // 获取标题和正文
            String contentName = list.pop();
            StringBuilder sb = new StringBuilder();
            String tmp;
            while (!list.isEmpty() && (tmp = list.peek()).startsWith(" ")) {
                sb.append(tmp.trim()).append("\r\n");
                list.pop();
            }
            // 放入数据库
            UploadBook uploadBook = new UploadBook();
            uploadBook.setBookId(bookId);
            uploadBook.setChapterId(bookId + i);
            uploadBook.setChapterName(contentName);
            uploadBook.setNum(chapterNum);

            uploadBook.setContent(sb.toString());
            uploadBook.setCreateTime(DateUtils.getNow());
            uploadBook.setUpdateTime(DateUtils.getNow());
            uploadBook.setWords(uploadBook.getContent().length());
            if (i <= defaultFreeChapterNum) {
                uploadBook.setReadAble(0);
            } else {
                uploadBook.setReadAble(1);
            }
            i++;
            chapterNum++;
            books.add(uploadBook);
            // 附加属性
            wordCount += uploadBook.getContent().length();
            if (firstChapterId == 0L) {
                firstChapterId = uploadBook.getChapterId();
                firstChapterName = uploadBook.getChapterName();
            }
            lastChapterId = uploadBook.getChapterId();
            lastChapterName = uploadBook.getChapterName();

            //存储
            if (books.size() >= 500) {
                uploadBookMapper.batchInsert(books);
                books.clear();
            }
        }
        if (books.size() > 0) {
            uploadBookMapper.batchInsert(books);
        }

        String fileName = file.getOriginalFilename();
        assert fileName != null;
        if (fileName.indexOf(".") > 0) {
            fileName = fileName.substring(0, fileName.lastIndexOf("."));
        }
        bookMapper.insertSelective(
                Book.builder()
                        .id(bookId)
                        .bookCategoryId(1)
                        .name(fileName)
                        .realname(fileName)
                        .author("作者")
                        .image("封面")
                        .description(content.toString())
                        .referralNum((byte) 5)
                        .price(100)
                        .chapterNum(chapterNum)
                        .wordCount(wordCount + "")
                        .firstChapterId(firstChapterId)
                        .firstChapterName(firstChapterName)
                        .lastChapterId(lastChapterId)
                        .lastChapterName(lastChapterName)
                        .lastChapterUtime(DateUtils.getNow())
                        .rank((byte) 0)
                        .checkRank((byte) 0)
                        .classifyWhiteList((byte) 0)
                        .putAdSet((byte) 0)
                        .build()
        );

    }

    @Override
    public Result<BookContent> getContentByContentId2(BookContentVO bookContentVO) {
        BookContent content = cacheService.getBookContentCache(bookContentVO.getBookId(), bookContentVO.getContentId());
        return Result.byObject(content);
    }

    @Autowired
    private EditBookMapper editBookMapper;

    @Override
    public Boolean updateBookContent(BookContentVO bookContentVO) {
        int i = editBookMapper.updateByExampleSelective(
                EditBook.builder()
                        .content(bookContentVO.getContent())
                        .updateTime(DateUtils.getNow())
                        .build()
                , EditBookExample.newAndCreateCriteria()
                        .andBookIdEqualTo(bookContentVO.getBookId())
                        .andChapterIdEqualTo(bookContentVO.getContentId())
                        .example()
                , EditBook.Column.content, EditBook.Column.updateTime
        );
        if (i <= 0) {
            i = editBookMapper.insert(EditBook.builder()
                    .bookId(bookContentVO.getBookId())
                    .chapterId(bookContentVO.getContentId())
                    .chapterName(bookContentVO.getContentName())
                    .content(bookContentVO.getContent())
                    .createTime(DateUtils.getNow())
                    .updateTime(DateUtils.getNow())
                    .build());
        }
        if (i > 0) {
            cacheService.updateEditBookCache(bookContentVO.getBookId(), bookContentVO.getContentId());
        }
        return i > 0;
    }

    public BookContent getBookContentFromDB(Long bookId, Long contentId) {
        UploadBook uploadBook = uploadBookMapper.selectOneByExampleSelective(UploadBookExample.newAndCreateCriteria()
                .andBookIdEqualTo(bookId)
                .andChapterIdEqualTo(contentId)
                .example(), UploadBook.Column.content, UploadBook.Column.chapterName);
        if (uploadBook == null) {
            return null;
        } else {
            return getBookContentFromVO(bookId, contentId, uploadBook.getChapterName(), uploadBook.getContent());
        }
    }

    public BookContent getBookContentFromVO(Long bookId, Long contenId, String contentName, String content) {
        BookContent bookContent = new BookContent();
        bookContent.setBookId(bookId + "");
        bookContent.setChapterId(contenId + "");
        bookContent.setName(contentName);
        bookContent.setContent(content);
        return bookContent;
    }


    @Autowired
    private RedisUtil redisUtil;

    /**
     * 获取默认免费章节数
     *
     * @return
     */
    public int getDefaultFreeChapterNum() {
        return cacheService.setBook_free_chapter_num();
    }

    public User getUserById(Long id) {
        User user = userMapper.selectByPrimaryKey(id);
        int freeCount = cacheService.getFreeCoinCache(id);
        if (user.getFreeKandian() != freeCount) {
            user.setFreeKandian(freeCount);
            // 更新免费看点
            User user1 = new User();
            user1.setId(user.getId());
            user1.setFreeKandian(freeCount);

            userMapper.updateByPrimaryKeySelective(user1);
        }
        return user;
    }

    private String getUserKey(Long id) {
        return getKey("user", id);
    }

    public String getKey(Object... keys) {
        StringBuilder sb = new StringBuilder();
        for (Object key : keys) {
            sb.append(key).append("_");
        }
        return sb.toString();
    }


    public EditBook getEditBookContent(Long bookId, Long contentId) {
        EditBook editBook = editBookMapper.selectOneByExampleWithBLOBs(EditBookExample.newAndCreateCriteria()
                .andBookIdEqualTo(bookId)
                .andChapterIdEqualTo(contentId)
                .example());
        if (editBook == null) {
            return null;
        }
        return editBook;
    }


    private List<BookRes> getRandom(List<BookRes> bookRes) {
        if (bookRes.size() <= 4) {
            return bookRes;
        }
        Collections.shuffle(bookRes);
        return bookRes.subList(0, 4);
    }

    /**
     * 签到或充值
     *
     * @param userId
     * @param type        充值类型:1=看点充值,2=VIP充值,3=系统操作看点,4=系统操作vip,5=签到
     * @param kandian
     * @param freeKandian
     * @return
     */
    @Override
    public int insertKandian(Long userId, String type, int kandian, int freeKandian) {
        // 签到看点
        rechargeMapper.insertSelective(
                Recharge.builder()
                        .userId(userId)
                        .type(type)
                        .kandian(kandian)
                        .freeKandian(freeKandian)
                        .remainFreeKandian(freeKandian)
                        .freeEndtime(DateUtils.get3DayAfter())
                        .createtime(DateUtils.getNow())
                        .updatetime(DateUtils.getNow())
                        .build()
        );
        int freeCount = getFreeCountFromCharges(getFreeCharges(userId));

        // 更新到用户
        User user = cacheService.getUserByIdCache(userId);
        User user1 = new User();
        user1.setId(userId);
        user1.setKandian(user.getKandian() + kandian);
        user1.setFreeKandian(freeCount);
        int i = userMapper.updateByPrimaryKeySelective(user1);
        if (i > 0) {
            //更新缓存
            cacheService.updateFreeCoinCache(userId);
        }
        return i;
    }

    @Override
    public Long getRecommendId() {
        BookExample example = BookExample.newAndCreateCriteria()
                .andIdBetween(Const.bookIdMin, Const.bookInMax)
                .example();
        example.orderBy(Book.Column.id.desc());
        Book book = bookMapper.selectOneByExampleSelective(example, Book.Column.id);
        if (book == null) {
            return Const.bookIdMin;
        }
        return book.getId() + 1;
    }

    @Override
    public int deleteUploadFile(Long id) {
        if (cacheService.isUploadBook(id)) {
            bookMapper.deleteByPrimaryKey(id);
            uploadBookMapper.deleteByExample(
                    UploadBookExample.newAndCreateCriteria()
                            .andBookIdEqualTo(id)
                            .example()
            );
            userRecentlyReadMapper.deleteByExample(
                    UserRecentlyReadExample.newAndCreateCriteria()
                            .andBookIdEqualTo(id)
                            .example()
            );
            return 1;
        } else {
            return 0;
        }
    }

    public Integer getFreeCoin(Long id) {
        return getFreeCountFromCharges(getFreeCharges(id));
    }

    public Integer getBook_guide_chapter_idx(String appId, Integer bookId) {
        Config config = configMapper.selectByName("book_guide_chapter_idx");
        return Integer.valueOf(config.getValue());
    }

    public String getQrcodeUrl(String appId) {
        AdminConfig adminConfig = adminConfigMapper.selectByAppid(appId);
        if (adminConfig == null) {
            return null;
        }
        String json = adminConfig.getJson().toString();
        AdminConfig.Info info = JsonUtils.getObject(json, AdminConfig.Info.class);
        return info.getAuthorizer_info().getQrcode_url();
    }
}
