package com.book.server.service.impl;

import com.book.dao.cps.entity.UserReadConfig;
import com.book.dao.cps.mapper.UserMapper;
import com.book.dao.cps.mapper.UserReadConfigMapper;
import com.book.dao.cps.pojo.User;
import com.book.dao.cps.pojo.example.UserReadConfigExample;
import com.book.dao.utils.DateUtils;
import com.book.server.service.CacheService;
import com.book.server.service.UserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

@Service
public class UserServiceImpl implements UserService {
    @Autowired
    private UserMapper userMapper;

    @Override
    public User getUserByOpenId(String openid) {
        User user1 = userMapper.selectByOpenId(openid);
        return user1;
    }

    @Override
    public User getUserByUserId(Long userId) {
        return userMapper.selectByPrimaryKey(userId);
    }

    @Autowired
    private UserReadConfigMapper userReadConfigMapper;

    @Override
    public UserReadConfig getUserReadConfigByUserId(Long userId) {
        UserReadConfig userReadConfig = userReadConfigMapper.selectOneByExample(
                UserReadConfigExample.newAndCreateCriteria()
                        .andUserIdEqualTo(userId)
                        .example()
        );
        return userReadConfig;
    }

    @Autowired
    private CacheService cacheService;

    @Override
    public Boolean setUserReadConfigByUserId(UserReadConfig readConfig) {
        readConfig.setUpdateTime(DateUtils.getNow());
        int i = userReadConfigMapper.updateByExampleSelective(
                readConfig,
                UserReadConfigExample.newAndCreateCriteria()
                        .andUserIdEqualTo(readConfig.getUserId())
                        .example()
        );
        if (i <= 0) {
            readConfig.setCreateTime(DateUtils.getNow());
            i = userReadConfigMapper.insert(readConfig);
        }
        cacheService.delUserReadConfigCache(readConfig.getUserId());
        return i > 0;
    }
}
