package com.book.server.task;

import com.book.dao.utils.DateUtils;
import com.book.dao.utils.TimeUtil;
import com.book.server.common.entity.Result;
import com.book.server.config.SunTypeEnum;
import com.book.server.service.SunDataService;
import com.book.server.vo.SunRequest;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.Calendar;

/**
 * created in 2021/9/28
 * Project: book-store
 *
 * @author win7
 */
@Component
@EnableScheduling
@Slf4j
public class SunDataTask {
    public static final Integer FIX_DELAY_HOUR = 1;

    @Autowired
    private SunDataService sunDataService;

    @Scheduled(cron = "0 0 0/1 * * ?")
    private void getData() {
        log.info("定时拉取sun数据");
        String timeFrom = TimeUtil.getDateStrAfter(-FIX_DELAY_HOUR, Calendar.HOUR_OF_DAY, TimeUtil.YYYY_M_DD_HH_MM_SS);
        String timeNow = TimeUtil.getCurrentDate(TimeUtil.YYYY_M_DD_HH_MM_SS);
        String dateNow = TimeUtil.getCurrentDate(TimeUtil.YYYYMMDD1);

        String channelId = "";


        for (SunTypeEnum sunTypeEnum : SunTypeEnum.values()) {
            SunRequest request = new SunRequest();

            switch (sunTypeEnum) {


                case user_collect:
                    request.setStart_time(timeFrom);
                    request.setEnd_time(timeNow);
                    sunDataService.requestData(request, sunTypeEnum);

                    break;

                case order:
                    request.setStart_time(timeFrom);
                    request.setEnd_time(timeNow);
                    request.setIs_finish("1");
                    sunDataService.requestData(request, sunTypeEnum);
                    break;

                default:
                    break;

            }


        }
    }
}
