package com.book.server.utils;

import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.net.ssl.HttpsURLConnection;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLSocketFactory;
import javax.servlet.http.HttpServletRequest;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.net.URLDecoder;
import java.net.URLEncoder;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;
import java.util.concurrent.atomic.AtomicInteger;

public class HttpUtils {
	private static Logger logger = LoggerFactory.getLogger(HttpUtils.class);
	//用于打印log
	private static AtomicInteger cnt = new AtomicInteger();
	
	public static String sendPost(String url, byte[] body){
		return sendPost(url, null, body,"other");
	}
	public static String sendPost(String url, byte[] body,String appid){
		return sendPost(url, null, body,appid);
	}
	public static String sendPost(String url, byte[] body,int timeout ,String appid){
		return sendPost(url, null, body, timeout ,appid);
	}
	public static String sendPostHttps(String url, byte[] body,String appid){
		return NetworkManager.request(url, body,appid);
	}
	public static String sendPost(String url, Map<String, String> params){
		return sendPost(url, params, 5000);
	}
	
	public static String sendPost2(String url, Map<String, String> params){
		return sendPost2(url, params, 5000);
	}
	public static String sendPost(String url,Map<String, String> heads, byte[] body){
		return sendPost(url,heads,body,"other");
	}
	public static String sendPost(String url,Map<String, String> heads, byte[] body,String appid){

		String result = "";
		HttpURLConnection httpURLConnection = null;
		InputStream inputStream = null;
		OutputStream outputStream = null;
		BufferedReader reader = null;
		StringBuffer buffer = new StringBuffer();
		try {
			URL u = new URL(url);
			httpURLConnection = (HttpURLConnection) u.openConnection();
			httpURLConnection.setRequestMethod("POST");
			httpURLConnection.setConnectTimeout(8000);
			httpURLConnection.setReadTimeout(30000);
			httpURLConnection.setDoInput(true);
			httpURLConnection.setDoOutput(true);
			httpURLConnection.setUseCaches(false);
			if(null != heads && !heads.isEmpty()){
				for (Entry<String, String> entry : heads.entrySet()) {
					httpURLConnection.setRequestProperty(entry.getKey(), entry.getValue());
				}
			}
			
			httpURLConnection.connect();
			
			outputStream = httpURLConnection.getOutputStream();
			
			outputStream.write(body);
			outputStream.flush();
			outputStream.close();
			inputStream = httpURLConnection.getInputStream();
			reader = new BufferedReader(new InputStreamReader(inputStream, "utf-8"));
			String line = null;
			while ((line = reader.readLine()) != null) {
				buffer.append(line);
			}
			result = buffer.toString();
		} catch (Exception e) {
			e.printStackTrace();
			logger.error("sendPost Exception", e);
			logger.error("HttpUtils#sendPost#body=" + new String(body));
		}
		finally {

			if(inputStream != null){
				try {
					inputStream.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			if(reader != null){
				try {
					reader.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			if(httpURLConnection != null){
				httpURLConnection.disconnect();
			}
		}
		return result;
	}
	public static String sendPost(String url,Map<String, String> heads, byte[] body,int timeout,String appid){

		String result = "";
		HttpURLConnection httpURLConnection = null;
		InputStream inputStream = null;
		OutputStream outputStream = null;
		BufferedReader reader = null;
		StringBuffer buffer = new StringBuffer();
		try {
			URL u = new URL(url);
			httpURLConnection = (HttpURLConnection) u.openConnection();
			httpURLConnection.setRequestMethod("POST");
			httpURLConnection.setConnectTimeout(timeout);
			httpURLConnection.setReadTimeout(timeout);
			//伪造User-Agent
//			httpURLConnection.setRequestProperty("User-Agent", "Mozilla/4.0 (compatible; MSIE 5.0; Windows NT; DigExt)");
			httpURLConnection.setDoInput(true);
			httpURLConnection.setDoOutput(true);
			httpURLConnection.setUseCaches(false);
			if(null != heads && !heads.isEmpty()){
				for (Entry<String, String> entry : heads.entrySet()) {
					httpURLConnection.setRequestProperty(entry.getKey(), entry.getValue());
				}
			}

			httpURLConnection.connect();

			outputStream = httpURLConnection.getOutputStream();

			outputStream.write(body);
			outputStream.flush();
			outputStream.close();
			inputStream = httpURLConnection.getInputStream();
			reader = new BufferedReader(new InputStreamReader(inputStream, "utf-8"));
			String line = null;
			while ((line = reader.readLine()) != null) {
				buffer.append(line);
			}
			result = buffer.toString();
		} catch (Exception e) {
			e.printStackTrace();
		}
		finally {

			if(inputStream != null){
				try {
					inputStream.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			if(reader != null){
				try {
					reader.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			if(httpURLConnection != null){
				httpURLConnection.disconnect();
			}
		}
		return result;
	}

	public static String sendPostHttps(String url,Map<String, String> heads, byte[] body) {
		String result = "";
		HttpsURLConnection httpsURLConnection = null;
		InputStream inputStream = null;
		OutputStream outputStream = null;
		BufferedReader reader = null;
		StringBuffer buffer = new StringBuffer();
		try {
	        javax.net.ssl.TrustManager[] tm = {  new MyX509TrustManager() };    
	        SSLContext sslContext = SSLContext.getInstance("SSL", "SunJSSE");    
	        sslContext.init(null,  tm, new java.security.SecureRandom());    
	          
	        // 从上述SSLContext对象中得到SSLSocketFactory对象     
	        SSLSocketFactory ssf = sslContext.getSocketFactory();    
	        //打开连接  
	        URL urlObj = new URL(url);    
	        httpsURLConnection = (HttpsURLConnection) urlObj.openConnection();    
	        httpsURLConnection.setSSLSocketFactory(ssf);    
			httpsURLConnection.setRequestMethod("POST");
			httpsURLConnection.setConnectTimeout(8000);
			httpsURLConnection.setReadTimeout(30000);
			httpsURLConnection.setDoInput(true);
			httpsURLConnection.setDoOutput(true);
			httpsURLConnection.setUseCaches(false);
			if(null != heads && !heads.isEmpty()){
				for (Entry<String, String> entry : heads.entrySet()) {
					httpsURLConnection.setRequestProperty(entry.getKey(), entry.getValue());
				}
			}
			
			httpsURLConnection.connect();
			
			outputStream = httpsURLConnection.getOutputStream();
			
			outputStream.write(body);
			outputStream.flush();
			outputStream.close();
			inputStream = httpsURLConnection.getInputStream();
			reader = new BufferedReader(new InputStreamReader(inputStream, "utf-8"));
			String line = null;
			while ((line = reader.readLine()) != null) {
				buffer.append(line);
			}
			result = buffer.toString();
		} catch (Exception e) {
			logger.error("sendPost Exception", e);
			logger.error("HttpUtils#sendPost#body=" + new String(body));
		} 
		finally {
			if(inputStream != null){
				try {
					inputStream.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			if(reader != null){
				try {
					reader.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			if(httpsURLConnection != null){
				httpsURLConnection.disconnect();
			}
		}
		return result;
	}	
	
	public static String sendPost(String url, Map<String, String> params,int timeOut){
		String result = "";
		HttpURLConnection httpURLConnection = null;
		InputStream inputStream = null;
		OutputStream outputStream = null;
		BufferedReader reader = null;
		StringBuffer buffer = new StringBuffer();
		try {
			URL u = new URL(url);
			httpURLConnection = (HttpURLConnection) u.openConnection();
			httpURLConnection.setRequestMethod("POST");
			httpURLConnection.setConnectTimeout(5000);
			httpURLConnection.setReadTimeout(timeOut);
			httpURLConnection.setDoInput(true);
			httpURLConnection.setDoOutput(true);
			httpURLConnection.setUseCaches(false);
			httpURLConnection.connect();
			
			outputStream = httpURLConnection.getOutputStream();
			
			StringBuffer buffer2 = new StringBuffer();
			Set<String> keys = params.keySet();
			for(String key : keys){
				buffer2.append(key);
				buffer2.append("=");
				String v = params.get(key);
				if(v == null || "".equals(v)){
					buffer2.append(v);
				}else{
					buffer2.append(URLEncoder.encode(params.get(key),"utf-8"));
				}
				buffer2.append("&");
			}
			
			String temp = buffer2.toString();
			String body = temp.substring(0, temp.length() - 1);
			logger.info("HttpUtils#sendPost#body=" + body);
			outputStream.write(body.getBytes("utf-8"));
			outputStream.flush();
			outputStream.close();
			inputStream = httpURLConnection.getInputStream();
			reader = new BufferedReader(new InputStreamReader(inputStream, "utf-8"));
			String line = null;
			while ((line = reader.readLine()) != null) {
				buffer.append(line);
			}
			result = buffer.toString();
		} catch (Exception e) {
			e.printStackTrace();
			return "";
		} 
		finally {
			if(inputStream != null){
				try {
					inputStream.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			if(reader != null){
				try {
					reader.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			if(httpURLConnection != null){
				httpURLConnection.disconnect();
			}
		}
		return result;
	}
	
	
	
	public static String sendGet(String url){
		CloseableHttpClient httpclient = HttpClients.createDefault();
//		String parameters = httpBuildQuery(data);
		long time = System.currentTimeMillis();
		int step = cnt.incrementAndGet();
//		System.out.println("["+parameters+"]");
//		logger.info(String.format("request%d:%s",step,parameters));
		String realUrl = url;
		logger.info("realUrl==="+realUrl);
		HttpGet httpget = new HttpGet(realUrl);  
		RequestConfig requestConfig = RequestConfig.custom()  
		        .setConnectTimeout(5000).setConnectionRequestTimeout(1000)  
		        .setSocketTimeout(30000).build(); 
		httpget.setConfig(requestConfig);
		String result = "";
		CloseableHttpResponse resp =null;
		try {
			resp = httpclient.execute(httpget);
			logger.info(String.format("time%d:%d",step,(System.currentTimeMillis()-time)));
			 // 获取响应实体    
            HttpEntity entity = resp.getEntity();
            result = EntityUtils.toString(entity);
            logger.info(String.format("reponse%d:%s",step,result));
		} catch (Exception e) {
			e.printStackTrace();
			logger.error(String.format("err%d:%s",step, url),e);
		}finally{
			try {
				if(resp != null) {
					resp.close();
				}
			} catch (Exception e) {
				e.printStackTrace();
				logger.error(String.format("close%d:%s",step, url),e);
			}
		}
         
        return result;
	}
	public static String sendGet(String url,String parameters){
		CloseableHttpClient httpclient = HttpClients.createDefault();
//		String parameters = httpBuildQuery(data);
		long time = System.currentTimeMillis();
		int step = cnt.incrementAndGet();
		System.out.println("["+parameters+"]");
		logger.info(String.format("request%d:%s",step,parameters));
		String realUrl = url+"?"+parameters;
		logger.info("realUrl==="+realUrl);
		HttpGet httpget = new HttpGet(realUrl);  
		RequestConfig requestConfig = RequestConfig.custom()  
				.setConnectTimeout(5000).setConnectionRequestTimeout(1000)  
				.setSocketTimeout(30000).build(); 
		httpget.setConfig(requestConfig);
		String result = "";
		CloseableHttpResponse resp =null;
		try {
			resp = httpclient.execute(httpget);
			logger.info(String.format("time%d:%d",step,(System.currentTimeMillis()-time)));
			// 获取响应实体    
			HttpEntity entity = resp.getEntity();
			result = EntityUtils.toString(entity);
			logger.info(String.format("reponse%d:%s",step,result));
		} catch (Exception e) {
			e.printStackTrace();
			logger.error(String.format("err%d:%s",step, url),e);
		}finally{
			try {
				if(resp != null) {
					resp.close();
				}
			} catch (Exception e) {
				e.printStackTrace();
				logger.error(String.format("close%d:%s",step, url),e);
			}
		}
		
		return result;
	}
	public static String sendGet(String url,Map<String, String> data){
		CloseableHttpClient httpclient = HttpClients.createDefault();
		String parameters = httpBuildQuery(data);
		long time = System.currentTimeMillis();
		int step = cnt.incrementAndGet();
		logger.info(String.format("request%d:%s",step,parameters));
		String realUrl = data.size()>0?url+"?"+parameters:url;
		logger.info("realUrl==="+realUrl);
		HttpGet httpget = new HttpGet(realUrl);  
		RequestConfig requestConfig = RequestConfig.custom()  
				.setConnectTimeout(5000).setConnectionRequestTimeout(1000)  
				.setSocketTimeout(30000).build(); 
		httpget.setConfig(requestConfig);
		String result = "";
		CloseableHttpResponse resp =null;
		try {
			resp = httpclient.execute(httpget);
			logger.info(String.format("time%d:%d",step,(System.currentTimeMillis()-time)));
			// 获取响应实体    
			HttpEntity entity = resp.getEntity();
			result = EntityUtils.toString(entity);
			logger.info(String.format("reponse%d:%s",step,result));
		} catch (Exception e) {
			e.printStackTrace();
			logger.error(String.format("err%d:%s",step, url),e);
		}finally{
			try {
				if(resp != null) {
					resp.close();
				}
			} catch (Exception e) {
				e.printStackTrace();
				logger.error(String.format("close%d:%s",step, url),e);
			}
		}
		
		return result;
	}
	
	public static String sendPost2(String url, Map<String, String> params,int timeOut){
		String result = "";
		HttpURLConnection httpURLConnection = null;
		InputStream inputStream = null;
		OutputStream outputStream = null;
		BufferedReader reader = null;
		StringBuffer buffer = new StringBuffer();
		try {
			URL u = new URL(url);
			httpURLConnection = (HttpURLConnection) u.openConnection();
			httpURLConnection.setRequestMethod("POST");
			httpURLConnection.setConnectTimeout(5000);
			httpURLConnection.setReadTimeout(timeOut);
			httpURLConnection.setDoInput(true);
			httpURLConnection.setDoOutput(true);
			httpURLConnection.setUseCaches(false);
			httpURLConnection.connect();
			
			outputStream = httpURLConnection.getOutputStream();
			
			StringBuffer buffer2 = new StringBuffer();
			Set<String> keys = params.keySet();
			for(String key : keys){
				buffer2.append(key);
				buffer2.append("=");
				String v = params.get(key);
				if(v == null || "".equals(v)){
					buffer2.append(v);
				}else{
					buffer2.append(params.get(key));
				}
				buffer2.append("&");
			}
			
			String temp = buffer2.toString();
			String body = temp.substring(0, temp.length() - 1);
			logger.info("HttpUtils#sendPost#body=" + body);
			outputStream.write(body.getBytes("utf-8"));
			outputStream.flush();
			outputStream.close();
			inputStream = httpURLConnection.getInputStream();
			reader = new BufferedReader(new InputStreamReader(inputStream, "utf-8"));
			String line = null;
			while ((line = reader.readLine()) != null) {
				buffer.append(line);
			}
			result = buffer.toString();
		} catch (Exception e) {
			e.printStackTrace();
			return "";
		} 
		finally {
			if(inputStream != null){
				try {
					inputStream.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			if(reader != null){
				try {
					reader.close();
				} catch (IOException e) {
					e.printStackTrace();
				}
			}
			if(httpURLConnection != null){
				httpURLConnection.disconnect();
			}
		}
		return result;
	}
	
	
	
	/**
	 *  执行POST请求
	 * @param url String 访问URL
	 * @param content json 字符串的请求报文
	 * @return String 响应报文
	 * @throws IOException 
	 * @throws ClientProtocolException 
	 * @throws Exception
	 *             异常
	 */
	public static String executePost(String url, String content) throws ClientProtocolException, IOException {
		DefaultHttpClient client = new DefaultHttpClient();
		HttpPost post = new HttpPost(url);
		try{
			post.setEntity(new StringEntity(content, "UTF-8"));
			HttpResponse response = client.execute(post);
			HttpEntity entity = response.getEntity();
			String rspMessage = EntityUtils.toString(entity, "UTF-8");
			EntityUtils.consume(entity);
			return rspMessage;
		}finally{
			client.getConnectionManager().shutdown();
		}
	}
	
	/**
	 * 参数编码
	 * @param data
	 * @return 
	 */
	public static String httpBuildQuery(Map<String, String> data) {
		if(data.size() == 0) {
			return "";
		}
		
		String ret = "";
		String k, v;
		Iterator<String> iterator = data.keySet().iterator();
		while (iterator.hasNext()) {
			k = iterator.next();
			v = data.get(k);
			if(v == null) {
				continue;
			}
			try {
				ret += URLEncoder.encode(k, "utf8") + "=" + URLEncoder.encode(v, "utf8");
			} catch (Exception e) {
				logger.error("err:"+k,e);
				continue;
			}
			ret += "&";
		}
		return ret.substring(0, ret.length() - 1);
	}
	
	public static String getRequestPostContent(HttpServletRequest request) {
		try {
			BufferedReader reader = request.getReader();
			StringBuilder reportBuilder = new StringBuilder();
			String tempStr = "";
			while ((tempStr = reader.readLine()) != null) {
				reportBuilder.append(tempStr);
			}
			return reportBuilder.toString();
		} catch (Exception e) {
			e.printStackTrace();
		}
		return null;
	}
	
	public static Map<String, String> getRequestDecodeParams(HttpServletRequest request){
		Map<String, String> map = new HashMap<String, String>();
		
		Map<String, String[]> params = request.getParameterMap();
		for (Entry<String, String[]> entry : params.entrySet()) {
			try {
				String val = URLDecoder.decode(entry.getValue()[0],"utf-8");
				map.put(entry.getKey(), val.equalsIgnoreCase("null")?"":val);
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			}
		}
		return map;
	}
	
	public static void main(String[] args) throws Exception {
	}




}
